# (c) 2021 Red Hat Inc.
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.

# Make coding more python3-ish

from __future__ import absolute_import, division, print_function


__metaclass__ = type

from textwrap import dedent

from ansible_collections.cisco.nxos.plugins.modules import nxos_snmp_server
from ansible_collections.cisco.nxos.tests.unit.compat.mock import patch

from .nxos_module import TestNxosModule, set_module_args


ignore_provider_arg = True


class TestNxosSnmpServerModule(TestNxosModule):

    module = nxos_snmp_server

    def setUp(self):
        super(TestNxosSnmpServerModule, self).setUp()

        self.mock_get_resource_connection = patch(
            "ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.resource_module_base.get_resource_connection",
        )
        self.get_resource_connection = self.mock_get_resource_connection.start()

        self.mock_get_config = patch(
            "ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.facts.snmp_server.snmp_server.Snmp_serverFacts.get_config",
        )
        self.get_config = self.mock_get_config.start()

    def tearDown(self):
        super(TestNxosSnmpServerModule, self).tearDown()
        self.get_resource_connection.stop()
        self.get_config.stop()

    def test_nxos_snmp_server_linear_merged(self):
        # test merged for linear attributes
        self.get_config.return_value = dedent(
            """\
            """,
        )
        set_module_args(
            dict(
                config=dict(
                    aaa_user=dict(cache_timeout=36000),
                    contact="testswitch@localhost",
                    context=dict(name="public", vrf="siteA", instance="test"),
                    counter=dict(cache=dict(timeout=1800)),
                    drop=dict(unknown_engine_id=True, unknown_user=True),
                    engine_id=dict(local="'00:00:00:63:00:01:00:10:20:15:10:03'"),
                    communities=[
                        dict(name="private", group="network-admin"),
                        dict(community="public", use_ipv4acl="myacl"),
                    ],
                    global_enforce_priv=True,
                    location="lab",
                    mib=dict(community_map=dict(community="public", context="public1")),
                    packetsize=484,
                    protocol=dict(enable=True),
                    source_interface=dict(informs="Ethernet1/1", traps="Ethernet1/2"),
                    system_shutdown=True,
                    tcp_session=dict(auth=True),
                ),
                state="merged",
            ),
            ignore_provider_arg,
        )
        commands = [
            "snmp-server community private group network-admin",
            "snmp-server community public use-ipv4acl myacl",
            "snmp-server globalEnforcePriv",
            "snmp-server tcp-session auth",
            "snmp-server counter cache timeout 1800",
            "snmp-server packetsize 484",
            "snmp-server drop unknown-user",
            "snmp-server source-interface informs Ethernet1/1",
            "snmp-server context public instance test vrf siteA",
            "snmp-server protocol enable",
            "snmp-server system-shutdown",
            "snmp-server aaa-user cache-timeout 36000",
            "snmp-server engineID local '00:00:00:63:00:01:00:10:20:15:10:03'",
            "snmp-server contact testswitch@localhost",
            "snmp-server drop unknown-engine-id",
            "snmp-server location lab",
            "snmp-server mib community-map public context public1",
            "snmp-server source-interface traps Ethernet1/2",
        ]
        result = self.execute_module(changed=True)
        self.assertEqual(set(result["commands"]), set(commands))

    def test_nxos_snmp_server_linear_merged_idempotent(self):
        # test merged for linear attributes
        self.get_config.return_value = dedent(
            """\
            snmp-server globalEnforcePriv
            snmp-server tcp-session auth
            snmp-server counter cache timeout 1800
            snmp-server packetsize 484
            snmp-server drop unknown-user
            snmp-server source-interface informs Ethernet1/1
            snmp-server context public vrf siteA
            snmp-server protocol enable
            snmp-server system-shutdown
            snmp-server aaa-user cache-timeout 36000
            snmp-server engineID local 00:00:00:63:00:01:00:10:20:15:10:03
            snmp-server contact testswitch@localhost
            snmp-server drop unknown-engine-id
            snmp-server location lab
            snmp-server mib community-map public context public1
            snmp-server source-interface traps Ethernet1/2
            """,
        )
        set_module_args(
            dict(
                config=dict(
                    aaa_user=dict(cache_timeout=36000),
                    contact="testswitch@localhost",
                    context=dict(name="public", vrf="siteA"),
                    counter=dict(cache=dict(timeout=1800)),
                    drop=dict(unknown_engine_id=True, unknown_user=True),
                    engine_id=dict(local="00:00:00:63:00:01:00:10:20:15:10:03"),
                    global_enforce_priv=True,
                    location="lab",
                    mib=dict(community_map=dict(community="public", context="public1")),
                    packetsize=484,
                    protocol=dict(enable=True),
                    source_interface=dict(informs="Ethernet1/1", traps="Ethernet1/2"),
                    system_shutdown=True,
                    tcp_session=dict(auth=True),
                ),
                state="merged",
            ),
            ignore_provider_arg,
        )
        result = self.execute_module(changed=False)
        self.assertEqual(result["commands"], [])

    def test_nxos_snmp_server_linear_replaced(self):
        # test replaced for linear attributes
        self.get_config.return_value = dedent(
            """\
            snmp-server globalEnforcePriv
            snmp-server tcp-session auth
            snmp-server counter cache timeout 1800
            snmp-server packetsize 484
            snmp-server drop unknown-user
            snmp-server source-interface informs Ethernet1/1
            snmp-server context public vrf siteA
            snmp-server protocol enable
            snmp-server system-shutdown
            snmp-server aaa-user cache-timeout 36000
            snmp-server engineID local 00:00:00:63:00:01:00:10:20:15:10:03
            snmp-server contact testswitch@localhost
            snmp-server drop unknown-engine-id
            snmp-server location lab
            snmp-server mib community-map public context public1
            snmp-server source-interface traps Ethernet1/2
            """,
        )
        set_module_args(
            dict(
                config=dict(
                    aaa_user=dict(cache_timeout=36500),
                    contact="testswitch@localhost",
                    context=dict(name="public", vrf="siteA"),
                    counter=dict(cache=dict(timeout=1860)),
                    engine_id=dict(local="00:00:00:63:00:01:00:10:20:15:10:03"),
                    global_enforce_priv=True,
                    location="lab",
                    mib=dict(community_map=dict(community="public", context="public1")),
                    packetsize=484,
                    protocol=dict(enable=True),
                    source_interface=dict(informs="Ethernet1/3", traps="Ethernet1/2"),
                    tcp_session=dict(auth=True),
                ),
                state="replaced",
            ),
            ignore_provider_arg,
        )
        commands = [
            "snmp-server counter cache timeout 1860",
            "no snmp-server drop unknown-user",
            "no snmp-server drop unknown-engine-id",
            "snmp-server source-interface informs Ethernet1/3",
            "no snmp-server system-shutdown",
            "snmp-server aaa-user cache-timeout 36500",
        ]
        result = self.execute_module(changed=True)
        self.assertEqual(set(result["commands"]), set(commands))

    def test_nxos_snmp_server_linear_overridden(self):
        # test overridden for linear attributes
        self.get_config.return_value = dedent(
            """\
            snmp-server globalEnforcePriv
            snmp-server tcp-session auth
            snmp-server counter cache timeout 1800
            snmp-server packetsize 484
            snmp-server drop unknown-user
            snmp-server source-interface informs Ethernet1/1
            snmp-server context public vrf siteA
            snmp-server protocol enable
            snmp-server system-shutdown
            snmp-server aaa-user cache-timeout 36000
            snmp-server engineID local 00:00:00:63:00:01:00:10:20:15:10:03
            snmp-server contact testswitch@localhost
            snmp-server drop unknown-engine-id
            snmp-server location lab
            snmp-server mib community-map public context public1
            snmp-server source-interface traps Ethernet1/2
            """,
        )
        set_module_args(
            dict(
                config=dict(
                    aaa_user=dict(cache_timeout=36500),
                    contact="testswitch@localhost",
                    context=dict(name="public", vrf="siteA"),
                    counter=dict(cache=dict(timeout=1860)),
                    engine_id=dict(local="00:00:00:63:00:01:00:10:20:15:10:03"),
                    global_enforce_priv=True,
                    location="lab",
                    mib=dict(community_map=dict(community="public", context="public1")),
                    packetsize=484,
                    protocol=dict(enable=True),
                    source_interface=dict(informs="Ethernet1/3", traps="Ethernet1/2"),
                    tcp_session=dict(auth=True),
                ),
                state="overridden",
            ),
            ignore_provider_arg,
        )
        commands = [
            "snmp-server counter cache timeout 1860",
            "no snmp-server drop unknown-user",
            "no snmp-server drop unknown-engine-id",
            "snmp-server source-interface informs Ethernet1/3",
            "no snmp-server system-shutdown",
            "snmp-server aaa-user cache-timeout 36500",
        ]
        result = self.execute_module(changed=True)
        self.assertEqual(set(result["commands"]), set(commands))

    def test_nxos_snmp_server_location_spaces(self):
        # test replaced for linear attributes
        self.get_config.return_value = dedent(
            """\
            snmp-server contact testswitch@localhost
            snmp-server location lab
            """,
        )
        set_module_args(
            dict(
                config=dict(
                    contact="test-switch @t localhost",
                    location="long/and.(complicated) address",
                ),
                state="replaced",
            ),
            ignore_provider_arg,
        )
        commands = [
            "snmp-server contact test-switch @t localhost",
            "snmp-server location long/and.(complicated) address",
        ]
        result = self.execute_module(changed=True)
        self.assertEqual(set(result["commands"]), set(commands))

    def test_nxos_snmp_server_traps_merged(self):
        # test merged for traps
        self.get_config.return_value = dedent(
            """\
            """,
        )
        set_module_args(
            dict(
                config=dict(
                    traps=dict(
                        aaa=dict(server_state_change=True),
                        bridge=dict(enable=True),
                        callhome=dict(event_notify=True, smtp_send_fail=True),
                        bgp=dict(enable=True),
                        ospf=dict(enable=True),
                        ospfv3=dict(enable=True),
                    ),
                ),
                state="merged",
            ),
            ignore_provider_arg,
        )
        commands = [
            "snmp-server enable traps aaa server-state-change",
            "snmp-server enable traps bridge newroot",
            "snmp-server enable traps bridge topologychange",
            "snmp-server enable traps callhome event-notify",
            "snmp-server enable traps callhome smtp-send-fail",
            "snmp-server enable traps bgp",
            "snmp-server enable traps ospf",
            "snmp-server enable traps ospfv3",
        ]
        result = self.execute_module(changed=True)
        self.assertEqual(set(result["commands"]), set(commands))

    def test_nxos_snmp_server_traps_merged_idempotent(self):
        # test merged for traps (idempotent)
        self.get_config.return_value = dedent(
            """\
            snmp-server enable traps aaa server-state-change
            snmp-server enable traps bridge newroot
            snmp-server enable traps bridge topologychange
            snmp-server enable traps callhome event-notify
            snmp-server enable traps callhome smtp-send-fail
            snmp-server enable traps bgp
            snmp-server enable traps ospf
            snmp-server enable traps ospfv3
            """,
        )
        set_module_args(
            dict(
                config=dict(
                    traps=dict(
                        aaa=dict(server_state_change=True),
                        bridge=dict(enable=True),
                        callhome=dict(event_notify=True, smtp_send_fail=True),
                        bgp=dict(enable=True),
                        ospf=dict(enable=True),
                        ospfv3=dict(enable=True),
                    ),
                ),
                state="merged",
            ),
            ignore_provider_arg,
        )

        result = self.execute_module(changed=False)
        self.assertEqual(result["commands"], [])

    def test_nxos_snmp_server_traps_replaced(self):
        # test replaced for traps
        self.get_config.return_value = dedent(
            """\
            snmp-server enable traps aaa server-state-change
            snmp-server enable traps bridge newroot
            snmp-server enable traps bridge topologychange
            snmp-server enable traps callhome event-notify
            snmp-server enable traps callhome smtp-send-fail
            snmp-server enable traps link cisco-xcvr-mon-status-chg
            """,
        )
        set_module_args(
            dict(
                config=dict(
                    traps=dict(
                        aaa=dict(server_state_change=True),
                        bridge=dict(enable=True),
                        cfs=dict(merge_failure=True),
                        link=dict(cisco_xcvr_mon_status_chg=False),
                    ),
                ),
                state="replaced",
            ),
            ignore_provider_arg,
        )
        commands = [
            "no snmp-server enable traps callhome event-notify",
            "no snmp-server enable traps callhome smtp-send-fail",
            "snmp-server enable traps cfs merge-failure",
            "no snmp-server enable traps link cisco-xcvr-mon-status-chg",
        ]
        result = self.execute_module(changed=True)
        self.assertEqual(set(result["commands"]), set(commands))

    def test_nxos_snmp_server_traps_replaced(self):
        # test replaced for traps
        self.get_config.return_value = dedent(
            """\
            snmp-server enable traps aaa server-state-change
            snmp-server enable traps bridge newroot
            snmp-server enable traps bridge topologychange
            snmp-server enable traps callhome event-notify
            snmp-server enable traps callhome smtp-send-fail
            """,
        )
        set_module_args(
            dict(
                config=dict(
                    traps=dict(
                        aaa=dict(server_state_change=True),
                        bridge=dict(enable=True),
                        cfs=dict(merge_failure=True),
                    ),
                ),
                state="replaced",
            ),
            ignore_provider_arg,
        )
        commands = [
            "no snmp-server enable traps callhome event-notify",
            "no snmp-server enable traps callhome smtp-send-fail",
            "snmp-server enable traps cfs merge-failure",
        ]
        result = self.execute_module(changed=True)
        self.assertEqual(set(result["commands"]), set(commands))

    def test_nxos_snmp_server_hosts_merged(self):
        # test merged for hosts
        self.get_config.return_value = dedent(
            """\
            """,
        )
        set_module_args(
            dict(
                config=dict(
                    hosts=[
                        dict(
                            host="192.168.1.1",
                            version="2c",
                            community="public",
                            traps=True,
                        ),
                        dict(host="192.168.1.1", source_interface="Ethernet1/1"),
                        dict(
                            host="192.168.2.1",
                            version="1",
                            community="private",
                            traps=True,
                        ),
                        dict(
                            host="192.168.2.1",
                            version="2c",
                            community="private",
                            informs=True,
                        ),
                        dict(
                            host="192.168.3.1",
                            version="3",
                            auth="private",
                            informs=True,
                            udp_port=65550,
                        ),
                        dict(
                            host="192.168.4.1",
                            version="3",
                            priv="private",
                            informs=True,
                        ),
                    ],
                ),
                state="merged",
            ),
            ignore_provider_arg,
        )
        commands = [
            "snmp-server host 192.168.2.1 informs version 2c private",
            "snmp-server host 192.168.1.1 traps version 2c public",
            "snmp-server host 192.168.4.1 informs version 3 priv private",
            "snmp-server host 192.168.1.1 source-interface Ethernet1/1",
            "snmp-server host 192.168.2.1 traps version 1 private",
            "snmp-server host 192.168.3.1 informs version 3 auth private udp-port 65550",
        ]
        result = self.execute_module(changed=True)
        self.assertEqual(set(result["commands"]), set(commands))

    def test_nxos_snmp_server_hosts_merged(self):
        # test merged for hosts
        self.get_config.return_value = dedent(
            """\
            snmp-server host 192.168.2.1 informs version 2c private
            snmp-server host 192.168.1.1 traps version 2c public
            snmp-server host 192.168.4.1 informs version 3 priv private
            snmp-server host 192.168.1.1 source-interface Ethernet1/1
            snmp-server host 192.168.2.1 traps version 1 private
            snmp-server host 192.168.3.1 informs version 3 auth private udp-port 65550
            """,
        )
        set_module_args(
            dict(
                config=dict(
                    hosts=[
                        dict(
                            host="192.168.1.1",
                            version="2c",
                            community="public",
                            traps=True,
                        ),
                        dict(host="192.168.1.1", source_interface="Ethernet1/1"),
                        dict(
                            host="192.168.2.1",
                            version="1",
                            community="private",
                            traps=True,
                        ),
                        dict(
                            host="192.168.2.1",
                            version="2c",
                            community="private",
                            informs=True,
                        ),
                        dict(
                            host="192.168.3.1",
                            version="3",
                            auth="private",
                            informs=True,
                            udp_port=65550,
                        ),
                        dict(
                            host="192.168.4.1",
                            version="3",
                            priv="private",
                            informs=True,
                        ),
                    ],
                ),
                state="merged",
            ),
            ignore_provider_arg,
        )
        result = self.execute_module(changed=False)
        self.assertEqual(result["commands"], [])

    def test_nxos_snmp_server_hosts_replaced(self):
        # test replaced for hosts
        self.get_config.return_value = dedent(
            """\
            snmp-server host 192.168.2.1 informs version 2c private
            snmp-server host 192.168.1.1 traps version 2c public
            snmp-server host 192.168.4.1 informs version 3 priv private
            snmp-server host 192.168.1.1 source-interface Ethernet1/1
            snmp-server host 192.168.2.1 traps version 1 private
            snmp-server host 192.168.3.1 informs version 3 auth private udp-port 65550
            """,
        )
        set_module_args(
            dict(
                config=dict(
                    hosts=[
                        dict(
                            host="192.168.1.1",
                            version="2c",
                            community="public",
                            traps=True,
                        ),
                        dict(host="192.168.1.1", source_interface="Ethernet1/1"),
                        dict(
                            host="192.168.2.1",
                            version="1",
                            community="private",
                            traps=True,
                        ),
                        dict(
                            host="192.168.2.1",
                            version="2c",
                            community="private",
                            informs=True,
                        ),
                        dict(host="192.168.2.1", filter_vrf="siteA"),
                        dict(host="192.168.4.1", use_vrf="siteB"),
                    ],
                ),
                state="replaced",
            ),
            ignore_provider_arg,
        )
        commands = [
            "no snmp-server host 192.168.4.1 informs version 3 priv private",
            "no snmp-server host 192.168.3.1 informs version 3 auth private udp-port 65550",
            "snmp-server host 192.168.2.1 filter-vrf siteA",
            "snmp-server host 192.168.4.1 use-vrf siteB",
        ]
        result = self.execute_module(changed=True)
        self.assertEqual(set(result["commands"]), set(commands))

    def test_nxos_snmp_server_users_merged(self):
        # test merged for users
        self.get_config.return_value = dedent(
            """\
            """,
        )
        set_module_args(
            dict(
                config=dict(
                    users=dict(
                        auth=[
                            dict(
                                user="snmp_user_1",
                                group="network-admin",
                                authentication=dict(
                                    algorithm="md5",
                                    password="0x5632724fb8ac3699296af26281e1d0f1",
                                    engine_id="1:1:1:1:1",
                                    localized_key=True,
                                ),
                            ),
                            dict(
                                user="snmp_user_2",
                                group="network-admin",
                                authentication=dict(
                                    algorithm="md5",
                                    password="0x5632724fb8ac3699296af26281e1d0f1",
                                    engine_id="2:2:2:2:2",
                                    priv=dict(
                                        privacy_password="0x5632724fb8ac3699296af26281e1d0f1",
                                    ),
                                    localizedv2_key=True,
                                ),
                            ),
                            dict(
                                user="snmp_user_3",
                                group="network-admin",
                                authentication=dict(
                                    algorithm="md5",
                                    password="0x5632724fb8ac3699296af26281e1d0f1",
                                    engine_id="3:3:3:3:3",
                                    priv=dict(
                                        privacy_password="0x5632724fb8ac3699296af26281e1d0f1",
                                        aes_128=True,
                                    ),
                                    localized_key=True,
                                ),
                            ),
                            dict(
                                user="snmp_user_4",
                                group="network-admin",
                                authentication=dict(
                                    algorithm="sha-256",
                                    password="0x5632724fb8ac3699296af26281e1d0f1",
                                    engine_id="4:4:4:4:4",
                                    priv=dict(
                                        privacy_password="0x5632724fb8ac3699296af26281e1d0f1",
                                        aes_128=True,
                                    ),
                                    localized_key=True,
                                ),
                            ),
                        ],
                    ),
                ),
                state="merged",
            ),
            ignore_provider_arg,
        )
        commands = [
            "snmp-server user snmp_user_2 network-admin auth md5 0x5632724fb8ac3699296af26281e1d0f1 priv 0x5632724fb8ac3699296af26281e1d0f1"
            " localizedkey engineID 2:2:2:2:2",
            "snmp-server user snmp_user_3 network-admin auth md5 0x5632724fb8ac3699296af26281e1d0f1 priv aes-128"
            " 0x5632724fb8ac3699296af26281e1d0f1 localizedV2key engineID 3:3:3:3:3",
            "snmp-server user snmp_user_1 network-admin auth md5 0x5632724fb8ac3699296af26281e1d0f1"
            " localizedkey engineID 1:1:1:1:1",
            "snmp-server user snmp_user_4 network-admin auth sha-256 0x5632724fb8ac3699296af26281e1d0f1 priv aes-128"
            " 0x5632724fb8ac3699296af26281e1d0f1 localizedkey engineID 4:4:4:4:4",
        ]
        result = self.execute_module(changed=True)
        self.assertEqual(set(result["commands"]), set(commands))

    def test_nxos_snmp_server_users_merged(self):
        # test merged for users
        self.get_config.return_value = dedent(
            """\
            snmp-server user user2 network-admin auth md5 0x5632724fb8ac3699296af262 priv 0x5632724fb8ac3699296af262 localizedV2key engineID 2:2:2:2:2
            snmp-server user user3 network-admin auth md5 0x5632724fb8ac3699296af262 priv aes-128 0x5632724fb8ac3699296af262 localizedkey engineID 3:3:3:3:3
            snmp-server user user1 network-admin auth md5 0x5632724fb8ac3699296af262 localizedkey engineID 1:1:1:1:1
            snmp-server user user4 network-admin auth sha-256 0x5632724fb8ac3699296af262 priv aes-128 0x5632724fb8ac3699296af262 localizedkey engineID 4:4:4:4:4
            """,
        )
        set_module_args(
            dict(
                config=dict(
                    users=dict(
                        auth=[
                            dict(
                                user="user1",
                                group="network-admin",
                                authentication=dict(
                                    algorithm="md5",
                                    password="0x5632724fb8ac3699296af262",
                                    engine_id="1:1:1:1:1",
                                    localized_key=True,
                                ),
                            ),
                            dict(
                                user="user2",
                                group="network-admin",
                                authentication=dict(
                                    algorithm="md5",
                                    password="0x5632724fb8ac3699296af262",
                                    engine_id="2:2:2:2:2",
                                    priv=dict(privacy_password="0x5632724fb8ac3699296af262"),
                                    localizedv2_key=True,
                                ),
                            ),
                            dict(
                                user="user3",
                                group="network-admin",
                                authentication=dict(
                                    algorithm="md5",
                                    password="0x5632724fb8ac3699296af262",
                                    engine_id="3:3:3:3:3",
                                    priv=dict(
                                        privacy_password="0x5632724fb8ac3699296af262",
                                        aes_128=True,
                                    ),
                                    localized_key=True,
                                ),
                            ),
                            dict(
                                user="user4",
                                group="network-admin",
                                authentication=dict(
                                    algorithm="sha-256",
                                    password="0x5632724fb8ac3699296af262",
                                    engine_id="4:4:4:4:4",
                                    priv=dict(
                                        privacy_password="0x5632724fb8ac3699296af262",
                                        aes_128=True,
                                    ),
                                    localized_key=True,
                                ),
                            ),
                        ],
                    ),
                ),
                state="merged",
            ),
            ignore_provider_arg,
        )
        result = self.execute_module(changed=False)
        self.assertEqual(result["commands"], [])

    def test_nxos_snmp_server_users_replaced(self):
        # test replaced for users
        self.get_config.return_value = dedent(
            """\
            snmp-server user user2 network-admin auth md5 0x5632724fb8ac3699296af262 priv 0x5632724fb8ac3699296af262 localizedkey engineID 2:2:2:2:2
            snmp-server user user3 network-admin auth md5 0x5632724fb8ac3699296af262 priv aes-128 0x5632724fb8ac3699296af262 localizedkey engineID 3:3:3:3:3
            snmp-server user user1 network-admin auth md5 0x5632724fb8ac3699296af262 localizedkey engineID 1:1:1:1:1
            """,
        )
        set_module_args(
            dict(
                config=dict(
                    users=dict(
                        auth=[
                            dict(
                                user="user1",
                                group="network-admin",
                                authentication=dict(
                                    algorithm="md5",
                                    password="0x5632724fb8ac3699296af262",
                                    engine_id="1:1:1:1:1",
                                    localized_key=True,
                                ),
                            ),
                            dict(
                                user="user4",
                                group="network-admin",
                                authentication=dict(
                                    algorithm="md5",
                                    password="0x5632724fb8ac3699296af262",
                                    engine_id="3:3:3:3:3",
                                    priv=dict(
                                        privacy_password="0x5632724fb8ac3699296af262",
                                        aes_128=True,
                                    ),
                                    localized_key=True,
                                ),
                            ),
                        ],
                    ),
                ),
                state="replaced",
            ),
            ignore_provider_arg,
        )
        commands = [
            "no snmp-server user user2 network-admin auth md5 0x5632724fb8ac3699296af262 priv"
            " 0x5632724fb8ac3699296af262 localizedkey engineID 2:2:2:2:2",
            "no snmp-server user user3 network-admin auth md5 0x5632724fb8ac3699296af262 priv"
            " aes-128 0x5632724fb8ac3699296af262 localizedkey engineID 3:3:3:3:3",
            "snmp-server user user4 network-admin auth md5 0x5632724fb8ac3699296af262 priv aes-128"
            " 0x5632724fb8ac3699296af262 localizedkey engineID 3:3:3:3:3",
        ]
        result = self.execute_module(changed=True)
        self.assertEqual(set(result["commands"]), set(commands))

    def test_nxos_snmp_server_deleted(self):
        # test deleted
        self.get_config.return_value = dedent(
            """\
            snmp-server globalEnforcePriv
            snmp-server tcp-session auth
            snmp-server counter cache timeout 1800
            snmp-server packetsize 484
            snmp-server drop unknown-user
            snmp-server source-interface informs Ethernet1/1
            snmp-server context public vrf siteA
            snmp-server protocol enable
            snmp-server system-shutdown
            snmp-server aaa-user cache-timeout 36000
            snmp-server engineID local 00:00:00:63:00:01:00:10:20:15:10:03
            snmp-server contact testswitch@localhost
            snmp-server drop unknown-engine-id
            snmp-server location lab
            snmp-server mib community-map public context public1
            snmp-server source-interface traps Ethernet1/2
            """,
        )
        set_module_args(dict(state="deleted"), ignore_provider_arg)
        commands = [
            "no snmp-server globalEnforcePriv",
            "no snmp-server tcp-session auth",
            "no snmp-server counter cache timeout 1800",
            "no snmp-server packetsize 484",
            "no snmp-server drop unknown-user",
            "no snmp-server source-interface informs Ethernet1/1",
            "no snmp-server context public vrf siteA",
            "no snmp-server protocol enable",
            "no snmp-server system-shutdown",
            "no snmp-server aaa-user cache-timeout 36000",
            "no snmp-server engineID local 00:00:00:63:00:01:00:10:20:15:10:03",
            "no snmp-server contact testswitch@localhost",
            "no snmp-server drop unknown-engine-id",
            "no snmp-server location lab",
            "no snmp-server mib community-map public context public1",
            "no snmp-server source-interface traps Ethernet1/2",
        ]
        result = self.execute_module(changed=True)
        self.assertEqual(set(result["commands"]), set(commands))

    def test_nxos_snmp_server_parsed(self):
        # test parsed
        set_module_args(
            dict(
                running_config=dedent(
                    """\
                    snmp-server community private group network-admin
                    snmp-server community public use-ipv4acl myacl
                    snmp-server globalEnforcePriv
                    snmp-server tcp-session auth
                    snmp-server counter cache timeout 1800
                    snmp-server packetsize 484
                    snmp-server drop unknown-user
                    snmp-server source-interface informs Ethernet1/1
                    snmp-server context public vrf siteA
                    snmp-server protocol enable
                    snmp-server system-shutdown
                    snmp-server aaa-user cache-timeout 36000
                    snmp-server engineID local 00:00:00:63:00:01:00:10:20:15:10:03
                    snmp-server contact testswitch@localhost
                    snmp-server drop unknown-engine-id
                    snmp-server location lab
                    snmp-server mib community-map public context public1
                    snmp-server source-interface traps Ethernet1/2
                    snmp-server user 1234 network-admin auth md5 0x7d425fbf09417c44bca69e1d9e9ce889 priv 0x7d425fbf09417c44bca69e1d9e9ce889 localizedkey
                    snmp-server user snmp_user_1 network-operator auth md5 0x5632724fb8ac3699296af26281e1d0f1 localizedkey
                    """,
                ),
                state="parsed",
            ),
            ignore_provider_arg,
        )

        parsed = dict(
            aaa_user=dict(cache_timeout=36000),
            contact="testswitch@localhost",
            communities=[
                dict(name="private", group="network-admin"),
                dict(name="public", use_ipv4acl="myacl"),
            ],
            context=dict(name="public", vrf="siteA"),
            counter=dict(cache=dict(timeout=1800)),
            drop=dict(unknown_engine_id=True, unknown_user=True),
            engine_id=dict(local="00:00:00:63:00:01:00:10:20:15:10:03"),
            global_enforce_priv=True,
            location="lab",
            mib=dict(community_map=dict(community="public", context="public1")),
            packetsize=484,
            protocol=dict(enable=True),
            source_interface=dict(informs="Ethernet1/1", traps="Ethernet1/2"),
            system_shutdown=True,
            tcp_session=dict(auth=True),
            users=dict(
                auth=[
                    dict(
                        user="1234",
                        group="network-admin",
                        authentication=dict(
                            algorithm="md5",
                            password="0x7d425fbf09417c44bca69e1d9e9ce889",
                            localized_key=True,
                            priv=dict(privacy_password="0x7d425fbf09417c44bca69e1d9e9ce889"),
                        ),
                    ),
                    dict(
                        user="snmp_user_1",
                        group="network-operator",
                        authentication=dict(
                            algorithm="md5",
                            password="0x5632724fb8ac3699296af26281e1d0f1",
                            localized_key=True,
                        ),
                    ),
                ],
            ),
        )
        result = self.execute_module(changed=False)
        self.assertEqual(result["parsed"], parsed)

    def test_nxos_snmp_server_rendered(self):
        # test rendered
        set_module_args(
            dict(
                config=dict(
                    aaa_user=dict(cache_timeout=36000),
                    contact="testswitch@localhost",
                    context=dict(name="public", vrf="siteA"),
                    counter=dict(cache=dict(timeout=1800)),
                    drop=dict(unknown_engine_id=True, unknown_user=True),
                    engine_id=dict(local="'00:00:00:63:00:01:00:10:20:15:10:03'"),
                    global_enforce_priv=True,
                    location="lab",
                    mib=dict(community_map=dict(community="public", context="public1")),
                    packetsize=484,
                    protocol=dict(enable=True),
                    source_interface=dict(informs="Ethernet1/1", traps="Ethernet1/2"),
                    system_shutdown=True,
                    tcp_session=dict(auth=True),
                ),
                state="rendered",
            ),
            ignore_provider_arg,
        )
        rendered = [
            "snmp-server globalEnforcePriv",
            "snmp-server tcp-session auth",
            "snmp-server counter cache timeout 1800",
            "snmp-server packetsize 484",
            "snmp-server drop unknown-user",
            "snmp-server source-interface informs Ethernet1/1",
            "snmp-server context public vrf siteA",
            "snmp-server protocol enable",
            "snmp-server system-shutdown",
            "snmp-server aaa-user cache-timeout 36000",
            "snmp-server engineID local '00:00:00:63:00:01:00:10:20:15:10:03'",
            "snmp-server contact testswitch@localhost",
            "snmp-server drop unknown-engine-id",
            "snmp-server location lab",
            "snmp-server mib community-map public context public1",
            "snmp-server source-interface traps Ethernet1/2",
        ]
        result = self.execute_module(changed=False)
        self.assertEqual(set(result["rendered"]), set(rendered))

    def test_nxos_snmp_server_gathered(self):
        # test gathered
        self.get_config.return_value = dedent(
            """\
            snmp-server globalEnforcePriv
            snmp-server tcp-session auth
            snmp-server counter cache timeout 1800
            snmp-server packetsize 484
            snmp-server drop unknown-user
            snmp-server source-interface informs Ethernet1/1
            snmp-server context public vrf siteA
            snmp-server protocol enable
            snmp-server system-shutdown
            snmp-server aaa-user cache-timeout 36000
            snmp-server engineID local 00:00:00:63:00:01:00:10:20:15:10:03
            snmp-server contact testswitch@localhost
            snmp-server drop unknown-engine-id
            snmp-server location lab
            snmp-server mib community-map public context public1
            snmp-server source-interface traps Ethernet1/2
            """,
        )
        set_module_args(dict(state="gathered"), ignore_provider_arg)

        gathered = dict(
            aaa_user=dict(cache_timeout=36000),
            contact="testswitch@localhost",
            context=dict(name="public", vrf="siteA"),
            counter=dict(cache=dict(timeout=1800)),
            drop=dict(unknown_engine_id=True, unknown_user=True),
            engine_id=dict(local="00:00:00:63:00:01:00:10:20:15:10:03"),
            global_enforce_priv=True,
            location="lab",
            mib=dict(community_map=dict(community="public", context="public1")),
            packetsize=484,
            protocol=dict(enable=True),
            source_interface=dict(informs="Ethernet1/1", traps="Ethernet1/2"),
            system_shutdown=True,
            tcp_session=dict(auth=True),
        )
        result = self.execute_module(changed=False)
        self.assertEqual(result["gathered"], gathered)

    def test_nxos_snmp_server_gathered_empty(self):
        self.get_config.return_value = dedent(
            """\
            """,
        )
        set_module_args(dict(state="gathered"), ignore_provider_arg)

        gathered = {}
        result = self.execute_module(changed=False)
        self.assertEqual(result["gathered"], gathered)



# -*- coding: utf-8 -*-
# Copyright (c) 2021, Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Make coding more python3-ish
from __future__ import absolute_import, division, print_function

__metaclass__ = type


import pytest

from ansible_collections.community.internal_test_tools.tests.unit.compat.mock import MagicMock, patch

from ansible_collections.community.dns.plugins.module_utils.provider import (
    ProviderInformation,
)

from ansible_collections.community.dns.plugins.module_utils.zone_record_api import (
    DNSAPIError,
)

from ansible_collections.community.dns.plugins.module_utils.module._utils import (
    normalize_dns_name,
    get_prefix,
)

from ..helper import (
    CustomProviderInformation,
)


def test_normalize_dns_name():
    assert normalize_dns_name('ExAMPLE.CoM.') == 'example.com'
    assert normalize_dns_name('EXAMpLE.CoM') == 'example.com'
    assert normalize_dns_name('Example.com') == 'example.com'
    assert normalize_dns_name('.') == ''
    assert normalize_dns_name(None) is None


def test_get_prefix():
    provider_information = CustomProviderInformation()
    provider_information.get_supported_record_types()
    assert get_prefix(
        normalized_zone='example.com', normalized_record='example.com', provider_information=provider_information) == ('example.com', None)
    assert get_prefix(
        normalized_zone='example.com', normalized_record='www.example.com', provider_information=provider_information) == ('www.example.com', 'www')
    assert get_prefix(normalized_zone='example.com', provider_information=provider_information) == ('example.com', None)
    assert get_prefix(normalized_zone='example.com', prefix='', provider_information=provider_information) == ('example.com', None)
    assert get_prefix(normalized_zone='example.com', prefix='.', provider_information=provider_information) == ('example.com', None)
    assert get_prefix(normalized_zone='example.com', prefix='www', provider_information=provider_information) == ('www.example.com', 'www')
    assert get_prefix(normalized_zone='example.com', prefix='www.', provider_information=provider_information) == ('www.example.com', 'www')
    assert get_prefix(normalized_zone='example.com', prefix='wWw.', provider_information=provider_information) == ('www.example.com', 'www')
    with pytest.raises(DNSAPIError):
        get_prefix(normalized_zone='example.com', normalized_record='example.org', provider_information=provider_information)
    with pytest.raises(DNSAPIError):
        get_prefix(normalized_zone='example.com', normalized_record='wwwexample.com', provider_information=provider_information)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "certified"}

DOCUMENTATION = r"""
---
module: aci_contract
short_description: Manage contract resources (vz:BrCP)
description:
- Manage Contract resources on Cisco ACI fabrics.
options:
  contract:
    description:
    - The name of the contract.
    type: str
    aliases: [ contract_name, name ]
  description:
    description:
    - Description for the contract.
    type: str
    aliases: [ descr ]
  tenant:
    description:
    - The name of the tenant.
    type: str
    aliases: [ tenant_name ]
  scope:
    description:
    - The scope of a service contract.
    - The APIC defaults to C(context) when unset during creation.
    type: str
    choices: [ application-profile, context, global, tenant ]
  priority:
    description:
    - The desired QoS class to be used.
    - The APIC defaults to C(unspecified) when unset during creation.
    type: str
    choices: [ level1, level2, level3, unspecified ]
  dscp:
    description:
    - The target Differentiated Service (DSCP) value.
    - The APIC defaults to C(unspecified) when unset during creation.
    type: str
    choices: [ AF11, AF12, AF13, AF21, AF22, AF23, AF31, AF32, AF33, AF41, AF42, AF43, CS0, CS1, CS2, CS3, CS4, CS5, CS6, CS7, EF, VA, unspecified ]
    aliases: [ target ]
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
  name_alias:
    description:
    - The alias for the current object. This relates to the nameAlias field in ACI.
    type: str
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation
- cisco.aci.owner

notes:
- This module does not manage Contract Subjects, see M(cisco.aci.aci_contract_subject) to do this.
  Contract Subjects can still be removed using this module.
- The C(tenant) used must exist before using this module in your playbook.
  The M(cisco.aci.aci_tenant) module can be used for this.
seealso:
- module: cisco.aci.aci_contract_subject
- module: cisco.aci.aci_tenant
- name: APIC Management Information Model reference
  description: More information about the internal APIC class B(vz:BrCP).
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Dag Wieers (@dagwieers)
"""

EXAMPLES = r"""
- name: Add a new contract
  cisco.aci.aci_contract:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: production
    contract: web_to_db
    description: Communication between web-servers and database
    scope: application-profile
    state: present
  delegate_to: localhost

- name: Remove an existing contract
  cisco.aci.aci_contract:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: production
    contract: web_to_db
    state: absent
  delegate_to: localhost

- name: Query a specific contract
  cisco.aci.aci_contract:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: production
    contract: web_to_db
    state: query
  delegate_to: localhost
  register: query_result

- name: Query all contracts
  cisco.aci.aci_contract:
    host: apic
    username: admin
    password: SomeSecretPassword
    state: query
  delegate_to: localhost
  register: query_result
"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec, aci_owner_spec


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(aci_owner_spec())
    argument_spec.update(
        contract=dict(type="str", aliases=["contract_name", "name"]),  # Not required for querying all objects
        tenant=dict(type="str", aliases=["tenant_name"]),  # Not required for querying all objects
        description=dict(type="str", aliases=["descr"]),
        scope=dict(type="str", choices=["application-profile", "context", "global", "tenant"]),
        priority=dict(type="str", choices=["level1", "level2", "level3", "unspecified"]),  # No default provided on purpose
        dscp=dict(
            type="str",
            choices=[
                "AF11",
                "AF12",
                "AF13",
                "AF21",
                "AF22",
                "AF23",
                "AF31",
                "AF32",
                "AF33",
                "AF41",
                "AF42",
                "AF43",
                "CS0",
                "CS1",
                "CS2",
                "CS3",
                "CS4",
                "CS5",
                "CS6",
                "CS7",
                "EF",
                "VA",
                "unspecified",
            ],
            aliases=["target"],
        ),  # No default provided on purpose
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
        name_alias=dict(type="str"),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["contract", "tenant"]],
            ["state", "present", ["contract", "tenant"]],
        ],
    )

    contract = module.params.get("contract")
    description = module.params.get("description")
    scope = module.params.get("scope")
    priority = module.params.get("priority")
    dscp = module.params.get("dscp")
    state = module.params.get("state")
    tenant = module.params.get("tenant")
    name_alias = module.params.get("name_alias")

    aci = ACIModule(module)
    aci.construct_url(
        root_class=dict(
            aci_class="fvTenant",
            aci_rn="tn-{0}".format(tenant),
            module_object=tenant,
            target_filter={"name": tenant},
        ),
        subclass_1=dict(
            aci_class="vzBrCP",
            aci_rn="brc-{0}".format(contract),
            module_object=contract,
            target_filter={"name": contract},
        ),
    )

    aci.get_existing()

    if state == "present":
        aci.payload(
            aci_class="vzBrCP",
            class_config=dict(
                name=contract,
                descr=description,
                scope=scope,
                prio=priority,
                targetDscp=dscp,
                nameAlias=name_alias,
            ),
        )

        aci.get_diff(aci_class="vzBrCP")

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



# (c) 2021 Red Hat Inc.
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.

# Make coding more python3-ish

from __future__ import absolute_import, division, print_function

__metaclass__ = type

from textwrap import dedent
from ansible_collections.cisco.iosxr.tests.unit.compat.mock import patch
from ansible_collections.cisco.iosxr.plugins.modules import (
    iosxr_bgp_neighbor_address_family,
)
from ansible_collections.cisco.iosxr.tests.unit.modules.utils import (
    set_module_args,
)
from .iosxr_module import TestIosxrModule


class TestIosxrBgpNeighborAddressFamilyModule(TestIosxrModule):
    module = iosxr_bgp_neighbor_address_family

    def setUp(self):
        super(TestIosxrBgpNeighborAddressFamilyModule, self).setUp()

        self.mock_get_resource_connection = patch(
            "ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.resource_module_base."
            "get_resource_connection"
        )
        self.get_resource_connection = (
            self.mock_get_resource_connection.start()
        )

        self.mock_get_config = patch(
            "ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.facts.bgp_neighbor_address_family.bgp_neighbor_address_family."
            "Bgp_neighbor_address_familyFacts.get_config"
        )
        self.get_config = self.mock_get_config.start()

    def tearDown(self):
        super(TestIosxrBgpNeighborAddressFamilyModule, self).tearDown()
        self.get_resource_connection.stop()
        self.get_config.stop()

    def test_iosxr_bgp_nbr_af_merged_idempotent(self):
        run_cfg = dedent(
            """\
                router bgp 1
                 bgp router-id 1.2.3.4
                 neighbor 1.1.1.1
                  remote-as 5
                  address-family ipv4 unicast
                   origin-as validation disable
                   bestpath origin-as allow invalid
                   aigp
                   aigp send med
                   send-community-ebgp
                   multipath
                   allowas-in 4
                   maximum-prefix 10 20 restart 10
                   as-override
                   capability orf prefix both
                   send-extended-community-ebgp
                   default-originate
                   next-hop-self
                   send-community-gshut-ebgp
                   soft-reconfiguration inbound
                   send-multicast-attributes
                   remove-private-AS inbound entire-aspath
                   route-policy test1 in
                   route-policy test1 out
                   next-hop-unchanged multipath
                 vrf vrf1
                  rd auto
                  address-family ipv4 unicast
                  neighbor 1.2.1.2
                   remote-as 5
                   address-family ipv4 unicast
                    multipath
                    capability orf prefix both
                    default-originate
            """
        )
        self.get_config.return_value = run_cfg
        set_module_args(
            dict(
                config=dict(
                    as_number="1",
                    vrfs=[
                        dict(
                            vrf="vrf1",
                            neighbors=[
                                dict(
                                    neighbor_address="1.2.1.2",
                                    address_family=[
                                        dict(
                                            afi="ipv4",
                                            safi="unicast",
                                            multipath=True,
                                            default_originate=dict(set=True),
                                            capability_orf_prefix="both",
                                        )
                                    ],
                                )
                            ],
                        )
                    ],
                    neighbors=[
                        dict(
                            neighbor_address="1.1.1.1",
                            address_family=[
                                dict(
                                    afi="ipv4",
                                    safi="unicast",
                                    multipath=True,
                                    default_originate=dict(set=True),
                                    capability_orf_prefix="both",
                                    send_multicast_attributes=dict(set=True),
                                    soft_reconfiguration=dict(
                                        inbound=dict(set=True)
                                    ),
                                    send_community_gshut_ebgp=dict(set=True),
                                    send_extended_community_ebgp=dict(
                                        set=True
                                    ),
                                    send_community_ebgp=dict(set=True),
                                    origin_as=dict(
                                        validation=dict(disable=True)
                                    ),
                                    remove_private_AS=dict(
                                        set=True,
                                        inbound=True,
                                        entire_aspath=True,
                                    ),
                                    route_policy=dict(
                                        inbound="test1", outbound="test1"
                                    ),
                                    maximum_prefix=dict(
                                        max_limit=10,
                                        threshold_value=20,
                                        restart=10,
                                    ),
                                    next_hop_self=dict(set=True),
                                    next_hop_unchanged=dict(multipath=True),
                                    aigp=dict(
                                        set=True, send_med=dict(set=True)
                                    ),
                                    as_override=dict(set=True),
                                    allowas_in=dict(value=4),
                                    bestpath_origin_as_allow_invalid=True,
                                    long_lived_graceful_restart=dict(
                                        stale_time=dict(send=20, accept=30)
                                    ),
                                )
                            ],
                        )
                    ],
                ),
                state="merged",
            )
        )
        self.execute_module(changed=False, commands=[])

    def test_iosxr_bgp_nbr_af_merged(self):
        self.maxDiff = None
        set_module_args(
            dict(
                config=dict(
                    as_number="1",
                    vrfs=[
                        dict(
                            vrf="vrf1",
                            neighbors=[
                                dict(
                                    neighbor_address="1.2.1.2",
                                    address_family=[
                                        dict(
                                            afi="ipv4",
                                            safi="unicast",
                                            multipath=True,
                                            default_originate=dict(set=True),
                                            capability_orf_prefix="both",
                                        )
                                    ],
                                )
                            ],
                        )
                    ],
                    neighbors=[
                        dict(
                            neighbor_address="1.1.1.1",
                            address_family=[
                                dict(
                                    afi="ipv4",
                                    safi="unicast",
                                    multipath=True,
                                    default_originate=dict(set=True),
                                    capability_orf_prefix="both",
                                    send_multicast_attributes=dict(set=True),
                                    soft_reconfiguration=dict(
                                        inbound=dict(set=True)
                                    ),
                                    send_community_gshut_ebgp=dict(set=True),
                                    send_extended_community_ebgp=dict(
                                        set=True
                                    ),
                                    send_community_ebgp=dict(set=True),
                                    origin_as=dict(
                                        validation=dict(disable=True)
                                    ),
                                    remove_private_AS=dict(
                                        set=True,
                                        inbound=True,
                                        entire_aspath=True,
                                    ),
                                    route_policy=dict(
                                        inbound="test1", outbound="test1"
                                    ),
                                    maximum_prefix=dict(
                                        max_limit=10,
                                        threshold_value=20,
                                        restart=10,
                                    ),
                                    next_hop_self=dict(set=True),
                                    next_hop_unchanged=dict(multipath=True),
                                    aigp=dict(
                                        set=True, send_med=dict(set=True)
                                    ),
                                    as_override=dict(set=True),
                                    bestpath_origin_as_allow_invalid=True,
                                    long_lived_graceful_restart=dict(
                                        stale_time=dict(send=20, accept=30)
                                    ),
                                )
                            ],
                        )
                    ],
                ),
                state="merged",
            )
        )
        commands = [
            "router bgp 1",
            "neighbor 1.1.1.1",
            "address-family ipv4 unicast",
            "aigp",
            "aigp send med",
            "as-override",
            "bestpath origin-as allow invalid",
            "capability orf prefix both",
            "default-originate",
            "maximum-prefix 10 20 restart 10",
            "multipath",
            "next-hop-self",
            "next-hop-unchanged multipath",
            "origin-as validation disable",
            "remove-private-AS inbound entire-aspath",
            "route-policy test1 in",
            "route-policy test1 out",
            "send-community-ebgp",
            "send-community-gshut-ebgp",
            "send-extended-community-ebgp",
            "send-multicast-attributes",
            "soft-reconfiguration inbound",
            "vrf vrf1",
            "neighbor 1.2.1.2",
            "address-family ipv4 unicast",
            "capability orf prefix both",
            "default-originate",
            "multipath",
        ]
        result = self.execute_module(changed=True)
        self.assertEqual(sorted(result["commands"]), sorted(commands))

    def test_iosxr_bgp_nbr_af_replaced(self):
        run_cfg = dedent(
            """\
                router bgp 1
                 bgp router-id 1.2.3.4
                 neighbor 1.1.1.1
                  remote-as 5
                  address-family ipv4 unicast
                   origin-as validation disable
                   bestpath origin-as allow invalid
                   aigp
                   aigp send med
                   send-community-ebgp
                   multipath
                   allowas-in 4
                   maximum-prefix 10 20 restart 10
                   as-override
                   capability orf prefix both
                   send-extended-community-ebgp
                   default-originate
                   next-hop-self
                   send-community-gshut-ebgp
                   soft-reconfiguration inbound
                   send-multicast-attributes
                   remove-private-AS inbound entire-aspath
                   next-hop-unchanged multipath
                 vrf vrf1
                  rd auto
                  address-family ipv4 unicast
                  neighbor 1.2.1.2
                   remote-as 5
                   address-family ipv4 unicast
                    multipath
                    capability orf prefix both
                    default-originate
                    route-policy test2 in
                    route-policy test2 out
            """
        )
        self.get_config.return_value = run_cfg
        set_module_args(
            dict(
                config=dict(
                    as_number="1",
                    vrfs=[
                        dict(
                            vrf="vrf1",
                            neighbors=[
                                dict(
                                    neighbor_address="1.2.1.2",
                                    address_family=[
                                        dict(
                                            afi="ipv4",
                                            safi="unicast",
                                            multipath=True,
                                            default_originate=dict(set=True),
                                            route_policy=dict(
                                                inbound="test1",
                                                outbound="test1",
                                            ),
                                        )
                                    ],
                                )
                            ],
                        )
                    ],
                ),
                state="replaced",
            )
        )
        commands = [
            "router bgp 1",
            "vrf vrf1",
            "neighbor 1.2.1.2",
            "address-family ipv4 unicast",
            "no capability orf prefix both",
            "route-policy test1 in",
            "route-policy test1 out",
        ]
        result = self.execute_module(changed=True)
        self.assertEqual(sorted(result["commands"]), sorted(commands))

    def test_iosxr_bgp_nbr_af_replaced_idempotent(self):
        run_cfg = dedent(
            """\
                router bgp 1
                 bgp router-id 1.2.3.4
                 neighbor 1.1.1.1
                  remote-as 5
                  address-family ipv4 unicast
                   origin-as validation disable
                   bestpath origin-as allow invalid
                   aigp
                   aigp send med
                   send-community-ebgp
                   multipath
                   allowas-in 4
                   maximum-prefix 10 20 restart 10
                   as-override
                   capability orf prefix both
                   send-extended-community-ebgp
                   default-originate
                   next-hop-self
                   send-community-gshut-ebgp
                   soft-reconfiguration inbound
                   send-multicast-attributes
                   remove-private-AS inbound entire-aspath
                   next-hop-unchanged multipath
                 vrf vrf1
                  rd auto
                  address-family ipv4 unicast
                  neighbor 1.2.1.2
                   remote-as 5
                   address-family ipv4 unicast
                    multipath
                    capability orf prefix both
                    default-originate
            """
        )
        self.get_config.return_value = run_cfg
        set_module_args(
            dict(
                config=dict(
                    as_number="1",
                    vrfs=[
                        dict(
                            vrf="vrf1",
                            neighbors=[
                                dict(
                                    neighbor_address="1.2.1.2",
                                    address_family=[
                                        dict(
                                            afi="ipv4",
                                            safi="unicast",
                                            multipath=True,
                                            default_originate=dict(set=True),
                                            capability_orf_prefix="both",
                                        )
                                    ],
                                )
                            ],
                        )
                    ],
                ),
                state="replaced",
            )
        )
        self.execute_module(changed=False, commands=[])

    def test_iosxr_bgp_nbr_af_deleted(self):
        self.maxDiff = None
        run_cfg = dedent(
            """\
                    router bgp 1
                     bgp router-id 1.2.3.4
                     neighbor 1.1.1.1
                      remote-as 5
                      address-family ipv4 unicast
                       origin-as validation disable
                       bestpath origin-as allow invalid
                       aigp
                       aigp send med
                       send-community-ebgp
                       multipath
                       allowas-in 4
                       maximum-prefix 10 20 restart 10
                       as-override
                       capability orf prefix both
                       send-extended-community-ebgp
                       default-originate
                       next-hop-self
                       send-community-gshut-ebgp
                       soft-reconfiguration inbound
                       send-multicast-attributes
                       remove-private-AS inbound entire-aspath
                       next-hop-unchanged multipath
                     vrf vrf1
                      rd auto
                      address-family ipv4 unicast
                      neighbor 1.2.1.2
                       remote-as 5
                       address-family ipv4 unicast
                        multipath
                        capability orf prefix both
                        default-originate
                """
        )
        self.get_config.return_value = run_cfg
        set_module_args(dict(config=dict(), state="deleted"))
        commands = [
            "router bgp 1",
            "neighbor 1.1.1.1",
            "no address-family ipv4 unicast",
            "vrf vrf1",
            "neighbor 1.2.1.2",
            "no address-family ipv4 unicast",
        ]
        result = self.execute_module(changed=True)
        self.assertEqual(sorted(result["commands"]), sorted(commands))

    def test_iosxr_bgp_nbr_af_deleted_idempotent(self):
        run_cfg = dedent(
            """\
            """
        )
        self.get_config.return_value = run_cfg
        set_module_args(dict(config=dict(as_number="1"), state="deleted"))

        result = self.execute_module(changed=False)
        self.assertEqual(result["commands"], [])

    def test_iosxr_bgp_nbr_af_rendered(self):
        set_module_args(
            dict(
                config=dict(
                    as_number="1",
                    vrfs=[
                        dict(
                            vrf="vrf1",
                            neighbors=[
                                dict(
                                    neighbor_address="1.2.1.2",
                                    address_family=[
                                        dict(
                                            afi="ipv4",
                                            safi="unicast",
                                            multipath=True,
                                            default_originate=dict(set=True),
                                            capability_orf_prefix="both",
                                        )
                                    ],
                                )
                            ],
                        )
                    ],
                    neighbors=[
                        dict(
                            neighbor_address="1.1.1.1",
                            address_family=[
                                dict(
                                    afi="ipv4",
                                    safi="unicast",
                                    multipath=True,
                                    default_originate=dict(set=True),
                                    capability_orf_prefix="both",
                                    send_multicast_attributes=dict(set=True),
                                    soft_reconfiguration=dict(
                                        inbound=dict(set=True)
                                    ),
                                    send_community_gshut_ebgp=dict(set=True),
                                    send_extended_community_ebgp=dict(
                                        set=True
                                    ),
                                    send_community_ebgp=dict(set=True),
                                    origin_as=dict(
                                        validation=dict(disable=True)
                                    ),
                                    remove_private_AS=dict(
                                        set=True,
                                        inbound=True,
                                        entire_aspath=True,
                                    ),
                                    maximum_prefix=dict(
                                        max_limit=10,
                                        threshold_value=20,
                                        restart=10,
                                    ),
                                    next_hop_self=dict(set=True),
                                    next_hop_unchanged=dict(multipath=True),
                                    aigp=dict(
                                        set=True, send_med=dict(set=True)
                                    ),
                                    as_override=dict(set=True),
                                    allowas_in=dict(value=4),
                                    bestpath_origin_as_allow_invalid=True,
                                    long_lived_graceful_restart=dict(
                                        stale_time=dict(send=20, accept=30)
                                    ),
                                )
                            ],
                        )
                    ],
                ),
                state="rendered",
            )
        )
        commands = [
            "router bgp 1",
            "neighbor 1.1.1.1",
            "address-family ipv4 unicast",
            "aigp",
            "aigp send med",
            "allowas-in 4",
            "as-override",
            "bestpath origin-as allow invalid",
            "capability orf prefix both",
            "default-originate",
            "maximum-prefix 10 20 restart 10",
            "multipath",
            "next-hop-self",
            "next-hop-unchanged multipath",
            "origin-as validation disable",
            "remove-private-AS inbound entire-aspath",
            "send-community-ebgp",
            "send-community-gshut-ebgp",
            "send-extended-community-ebgp",
            "send-multicast-attributes",
            "soft-reconfiguration inbound",
            "vrf vrf1",
            "neighbor 1.2.1.2",
            "address-family ipv4 unicast",
            "capability orf prefix both",
            "default-originate",
            "multipath",
        ]

        result = self.execute_module(changed=False)
        self.assertEqual(sorted(result["rendered"]), sorted(commands))

    def test_iosxr_bgp_global_parsed(self):
        self.maxDiff = None
        set_module_args(
            dict(
                running_config="router bgp 1\n bgp router-id 1.2.1.3\n neighbor 1.1.1.1\n  "
                "remote-as 6\n  address-family ipv4 unicast\n   origin-as validation disable\n   "
                "bestpath origin-as allow invalid\n   weight 0\n   send-community-ebgp\n  "
                " multipath\n   allowas-in 3\n   maximum-prefix 1 1 discard-extra-paths\n"
                "   capability orf prefix both\n   "
                "send-extended-community-ebgp\n   long-lived-graceful-restart capable\n"
                "   next-hop-self\n   "
                "remove-private-AS\n   send-community-gshut-ebgp inheritance-disable\n "
                "  send-multicast-attributes\n   "
                "remove-private-AS inbound entire-aspath\n   "
                "next-hop-unchanged multipath\n  !\n  "
                "!\n !",
                state="parsed",
            )
        )
        result = self.execute_module(changed=False)
        parsed_list = {
            "as_number": "1",
            "neighbors": [
                {
                    "address_family": [
                        {
                            "afi": "ipv4",
                            "allowas_in": {"value": 3},
                            "bestpath_origin_as_allow_invalid": True,
                            "capability_orf_prefix": "both",
                            "long_lived_graceful_restart": {"capable": True},
                            "multipath": True,
                            "next_hop_self": {"set": True},
                            "next_hop_unchanged": {"multipath": True},
                            "origin_as": {"validation": {"disable": True}},
                            "remove_private_AS": {
                                "entire_aspath": True,
                                "inbound": True,
                                "set": True,
                            },
                            "safi": "unicast",
                            "send_community_ebgp": {"set": True},
                            "send_community_gshut_ebgp": {
                                "inheritance_disable": True
                            },
                            "send_extended_community_ebgp": {"set": True},
                            "send_multicast_attributes": {"set": True},
                            "weight": 0,
                        }
                    ],
                    "neighbor_address": "1.1.1.1",
                }
            ],
        }

        self.assertEqual(parsed_list, result["parsed"])

    def test_iosxr_bgp_nbr_af_overridden(self):
        run_cfg = dedent(
            """\
                router bgp 1
                 bgp router-id 1.2.3.4
                 neighbor 1.1.1.1
                  remote-as 5
                  address-family ipv4 unicast
                   origin-as validation disable
                   bestpath origin-as allow invalid
                   aigp
                   aigp send med
                   send-community-ebgp
                   multipath
                   allowas-in 4
                   maximum-prefix 10 20 restart 10
                   as-override
                   capability orf prefix both
                   send-extended-community-ebgp
                   default-originate
                   next-hop-self
                   send-community-gshut-ebgp
                   soft-reconfiguration inbound
                   send-multicast-attributes
                   remove-private-AS inbound entire-aspath
                   next-hop-unchanged multipath
                 vrf vrf1
                  rd auto
                  address-family ipv4 unicast
                  neighbor 1.2.1.2
                   remote-as 5
                   address-family ipv4 unicast
                    multipath
                    capability orf prefix both
                    default-originate
                    route-policy test2 in
                    route-policy test2 out
            """
        )
        self.get_config.return_value = run_cfg
        set_module_args(
            dict(
                config=dict(
                    as_number="1",
                    vrfs=[
                        dict(
                            vrf="vrf1",
                            neighbors=[
                                dict(
                                    neighbor_address="1.2.1.2",
                                    address_family=[
                                        dict(
                                            afi="ipv4",
                                            safi="unicast",
                                            multipath=True,
                                            default_originate=dict(set=True),
                                        )
                                    ],
                                )
                            ],
                        )
                    ],
                ),
                state="overridden",
            )
        )
        commands = [
            "router bgp 1",
            "vrf vrf1",
            "neighbor 1.2.1.2",
            "address-family ipv4 unicast",
            "no capability orf prefix both",
            "no route-policy test2 in",
            "no route-policy test2 out",
            "neighbor 1.1.1.1",
            "no address-family ipv4 unicast",
        ]
        result = self.execute_module(changed=True)
        self.assertEqual(sorted(result["commands"]), sorted(commands))



#!/usr/bin/python
# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = r'''
---
module: ec2_vpc_vgw_info
version_added: 1.0.0
short_description: Gather information about virtual gateways in AWS
description:
    - Gather information about virtual gateways in AWS.
options:
  filters:
    description:
      - A dict of filters to apply. Each dict item consists of a filter key and a filter value.
        See U(https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_DescribeVpnGateways.html) for possible filters.
    type: dict
  vpn_gateway_ids:
    description:
      - Get details of a specific Virtual Gateway ID. This value should be provided as a list.
    type: list
    elements: str
author: "Nick Aslanidis (@naslanidis)"
extends_documentation_fragment:
- amazon.aws.aws
- amazon.aws.ec2

'''

EXAMPLES = r'''
# # Note: These examples do not set authentication details, see the AWS Guide for details.

- name: Gather information about all virtual gateways for an account or profile
  community.aws.ec2_vpc_vgw_info:
    region: ap-southeast-2
    profile: production
  register: vgw_info

- name: Gather information about a filtered list of Virtual Gateways
  community.aws.ec2_vpc_vgw_info:
    region: ap-southeast-2
    profile: production
    filters:
        "tag:Name": "main-virt-gateway"
  register: vgw_info

- name: Gather information about a specific virtual gateway by VpnGatewayIds
  community.aws.ec2_vpc_vgw_info:
    region: ap-southeast-2
    profile: production
    vpn_gateway_ids: vgw-c432f6a7
  register: vgw_info
'''

RETURN = r'''
virtual_gateways:
    description: The virtual gateways for the account.
    returned: always
    type: list
    sample: [
        {
            "state": "available",
            "tags": [
                {
                    "key": "Name",
                    "value": "TEST-VGW"
                }
            ],
            "type": "ipsec.1",
            "vpc_attachments": [
                {
                    "state": "attached",
                    "vpc_id": "vpc-22a93c74"
                }
            ],
            "vpn_gateway_id": "vgw-23e3d64e"
        }
    ]

changed:
    description: True if listing the virtual gateways succeeds.
    returned: always
    type: bool
    sample: "false"
'''

try:
    import botocore
except ImportError:
    pass  # Handled by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.core import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import ansible_dict_to_boto3_filter_list


def get_virtual_gateway_info(virtual_gateway):
    virtual_gateway_info = {'VpnGatewayId': virtual_gateway['VpnGatewayId'],
                            'State': virtual_gateway['State'],
                            'Type': virtual_gateway['Type'],
                            'VpcAttachments': virtual_gateway['VpcAttachments'],
                            'Tags': virtual_gateway.get('Tags', [])}
    return virtual_gateway_info


def list_virtual_gateways(client, module):
    params = dict()

    params['Filters'] = ansible_dict_to_boto3_filter_list(module.params.get('filters'))
    params['DryRun'] = module.check_mode

    if module.params.get("vpn_gateway_ids"):
        params['VpnGatewayIds'] = module.params.get("vpn_gateway_ids")

    try:
        all_virtual_gateways = client.describe_vpn_gateways(**params)
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Failed to list gateways")

    return [camel_dict_to_snake_dict(get_virtual_gateway_info(vgw))
            for vgw in all_virtual_gateways['VpnGateways']]


def main():
    argument_spec = dict(
        filters=dict(type='dict', default=dict()),
        vpn_gateway_ids=dict(type='list', default=None, elements='str')
    )

    module = AnsibleAWSModule(argument_spec=argument_spec, supports_check_mode=True)

    try:
        connection = module.client('ec2')
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg='Failed to connect to AWS')

    # call your function here
    results = list_virtual_gateways(connection, module)

    module.exit_json(virtual_gateways=results)


if __name__ == '__main__':
    main()



# (c) 2020 Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

# Make coding more python3-ish
from __future__ import absolute_import, division, print_function

__metaclass__ = type

from unittest.mock import MagicMock, patch

import pytest
from ansible.errors import AnsibleError, AnsibleFileNotFound
from ansible.module_utils._text import to_bytes
from ansible.playbook.play_context import PlayContext
from ansible.plugins.loader import connection_loader
from ansible_collections.ansible.netcommon.plugins.connection import libssh

pylibsshext = pytest.importorskip("pylibsshext")


@pytest.fixture(name="conn")
def plugin_fixture():
    pc = PlayContext()
    pc.port = 8080
    pc.timeout = 60

    conn = connection_loader.get("ansible.netcommon.libssh", pc, "/dev/null")
    return conn


def test_libssh_connect(conn, monkeypatch):
    """Test the libssh connection plugin.

    :param monkeypatch: pytest fixture
    """
    conn.set_options(
        direct={
            "remote_addr": "localhost",
            "remote_user": "user1",
            "password": "test",
            "host_key_checking": False,
        }
    )

    mock_session = MagicMock()
    monkeypatch.setattr(libssh, "Session", mock_session)
    mock_ssh = MagicMock()
    mock_session.return_value = mock_ssh
    conn._connect()
    mock_ssh.connect.assert_called_with(
        host="localhost",
        host_key_checking=False,
        look_for_keys=True,
        password="test",
        port=8080,
        timeout=60,
        user="user1",
        private_key=None,
    )


def test_libssh_close(conn):
    conn.ssh = MagicMock()
    conn.sftp = MagicMock()
    conn.chan = MagicMock()

    conn.close()

    conn.sftp.close.assert_called_with()
    conn.chan.close.assert_called_with()
    conn.ssh.close.assert_called_with()


@patch("ansible.plugins.connection.ConnectionBase.exec_command")
def test_libssh_exec_command(mocked_super, conn):
    with pytest.raises(AnsibleError):
        conn.exec_command(cmd="ls", in_data=True)

    mock_chan = MagicMock()
    mock_chan.request_shell = MagicMock()
    mock_chan.exec_command = MagicMock()
    mock_chan.exec_command.return_value = MagicMock(
        returncode=0, stdout="echo hello", stderr=""
    )

    attr = {"new_channel.return_value": mock_chan}
    mock_ssh = MagicMock(**attr)
    conn.ssh = mock_ssh

    rc, out, err = conn.exec_command(cmd="echo hello")

    assert (rc, out, err) == (0, "echo hello", "")


@patch("ansible.plugins.connection.ConnectionBase.put_file")
def test_libssh_put_file_not_exist(mocked_super, conn):
    with pytest.raises(AnsibleFileNotFound):
        conn.put_file(in_path="", out_path="")


@patch("os.path.exists")
@patch("ansible.plugins.connection.ConnectionBase.put_file")
def test_libssh_put_file(mocked_super, mock_exists, conn):
    mock_sftp = MagicMock()
    attr = {"sftp.return_value": mock_sftp}
    mock_ssh = MagicMock(**attr)
    conn.ssh = mock_ssh

    file_path = "test_libssh.py"
    conn.put_file(in_path=file_path, out_path=file_path)
    mock_sftp.put.assert_called_with(to_bytes(file_path), to_bytes(file_path))


@patch("ansible.plugins.connection.ConnectionBase.fetch_file")
def test_libssh_fetch_file(mocked_super, conn, monkeypatch):
    mock_session = MagicMock()
    monkeypatch.setattr(libssh, "Session", mock_session)
    mock_ssh = MagicMock()
    mock_session.return_value = mock_ssh

    file_path = "test_libssh.py"
    conn.fetch_file(in_path=file_path, out_path=file_path)

    conn.sftp.get.assert_called_with(
        to_bytes(file_path),
        to_bytes(file_path),
    )



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright: (c) 2018, Ansible Project
# Copyright: (c) 2018, Abhijeet Kasurde <akasurde@redhat.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type


DOCUMENTATION = r"""
---
module: digital_ocean_certificate_info
short_description: Gather information about DigitalOcean certificates
description:
    - This module can be used to gather information about DigitalOcean provided certificates.
    - This module was called C(digital_ocean_certificate_facts) before Ansible 2.9. The usage did not change.
author: "Abhijeet Kasurde (@Akasurde)"
options:
  certificate_id:
    description:
     - Certificate ID that can be used to identify and reference a certificate.
    required: false
    type: str
requirements:
  - "python >= 2.6"
extends_documentation_fragment:
- community.digitalocean.digital_ocean.documentation

"""


EXAMPLES = r"""
- name: Gather information about all certificates
  community.digitalocean.digital_ocean_certificate_info:
    oauth_token: "{{ oauth_token }}"

- name: Gather information about certificate with given id
  community.digitalocean.digital_ocean_certificate_info:
    oauth_token: "{{ oauth_token }}"
    certificate_id: "892071a0-bb95-49bc-8021-3afd67a210bf"

- name: Get not after information about certificate
  community.digitalocean.digital_ocean_certificate_info:
  register: resp_out
- set_fact:
    not_after_date: "{{ item.not_after }}"
  loop: "{{ resp_out.data | community.general.json_query(name) }}"
  vars:
    name: "[?name=='web-cert-01']"
- debug:
    var: not_after_date
"""


RETURN = r"""
data:
    description: DigitalOcean certificate information
    returned: success
    type: list
    elements: dict
    sample: [
        {
          "id": "892071a0-bb95-49bc-8021-3afd67a210bf",
          "name": "web-cert-01",
          "not_after": "2017-02-22T00:23:00Z",
          "sha1_fingerprint": "dfcc9f57d86bf58e321c2c6c31c7a971be244ac7",
          "created_at": "2017-02-08T16:02:37Z"
        },
    ]
"""

from traceback import format_exc
from ansible.module_utils.basic import AnsibleModule
from ansible_collections.community.digitalocean.plugins.module_utils.digital_ocean import (
    DigitalOceanHelper,
)
from ansible.module_utils._text import to_native


def core(module):
    certificate_id = module.params.get("certificate_id", None)
    rest = DigitalOceanHelper(module)

    base_url = "certificates"
    if certificate_id is not None:
        response = rest.get("%s/%s" % (base_url, certificate_id))
        status_code = response.status_code

        if status_code != 200:
            module.fail_json(msg="Failed to retrieve certificates for DigitalOcean")

        certificate = [response.json["certificate"]]
    else:
        certificate = rest.get_paginated_data(
            base_url=base_url + "?", data_key_name="certificates"
        )

    module.exit_json(changed=False, data=certificate)


def main():
    argument_spec = DigitalOceanHelper.digital_ocean_argument_spec()
    argument_spec.update(
        certificate_id=dict(type="str", required=False),
    )
    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=True)
    if module._name in (
        "digital_ocean_certificate_facts",
        "community.digitalocean.digital_ocean_certificate_facts",
    ):
        module.deprecate(
            "The 'digital_ocean_certificate_facts' module has been renamed to 'digital_ocean_certificate_info'",
            version="2.0.0",
            collection_name="community.digitalocean",
        )  # was Ansible 2.13

    try:
        core(module)
    except Exception as e:
        module.fail_json(msg=to_native(e), exception=format_exc())


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: mnt_session_delete_all
short_description: Resource module for MNT Session Delete All
description:
- Manage operation delete of the resource MNT Session Delete All.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.ise.module
author: Rafael Campos (@racampos)
options: {}
requirements:
- ciscoisesdk >= 2.0.3
- python >= 3.5
notes:
  - SDK Method used are
    misc.Misc.delete_all_sessions,

  - Paths used are
    delete /Session/Delete/All,

"""

EXAMPLES = r"""
- name: Delete all
  cisco.ise.mnt_session_delete_all:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: dict
  sample: >
    {}
"""



# (c) 2021 Red Hat Inc.
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.

# Make coding more python3-ish

from __future__ import absolute_import, division, print_function

from typing import Sequence


__metaclass__ = type

from textwrap import dedent

from ansible_collections.cisco.nxos.plugins.modules import nxos_prefix_lists
from ansible_collections.cisco.nxos.tests.unit.compat.mock import patch
from ansible_collections.cisco.nxos.tests.unit.modules.utils import AnsibleFailJson

from .nxos_module import TestNxosModule, set_module_args


ignore_provider_arg = True


class TestNxosPrefixListsModule(TestNxosModule):

    # Testing strategy
    # ------------------
    # (a) The unit tests cover `merged` and `replaced` for every attribute.
    #     Since `overridden` is essentially `replaced` but at a larger
    #     scale, these indirectly cover `overridden` as well.
    # (b) For linear attributes replaced is not valid and hence, those tests
    #     delete the attributes from the config subsection.
    # (c) The argspec for VRFs is same as the top-level spec and the config logic
    #     is re-used. Hence, those attributes are not explictly covered. However, a
    #     combination of VRF + top-level spec + AF is tested.

    module = nxos_prefix_lists

    def setUp(self):
        super(TestNxosPrefixListsModule, self).setUp()

        self.mock_get_resource_connection = patch(
            "ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.resource_module_base.get_resource_connection",
        )
        self.get_resource_connection = self.mock_get_resource_connection.start()

        self.mock_get_config = patch(
            "ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.facts.prefix_lists.prefix_lists.Prefix_listsFacts.get_config",
        )
        self.get_config = self.mock_get_config.start()

    def tearDown(self):
        super(TestNxosPrefixListsModule, self).tearDown()
        self.get_resource_connection.stop()
        self.get_config.stop()

    def test_nxos_prefix_lists_linear_merged(self):
        # test merged for linear attributes
        self.get_config.return_value = dedent(
            """\
            """,
        )
        set_module_args(
            dict(
                config=[
                    dict(
                        afi="ipv4",
                        prefix_lists=[
                            dict(
                                name="plist1",
                                description="Test plist1",
                                entries=[
                                    dict(
                                        sequence=10,
                                        action="permit",
                                        prefix="192.168.1.0/24",
                                    ),
                                    dict(
                                        sequence=20,
                                        action="deny",
                                        prefix="192.168.2.0/24",
                                        mask="255.255.255.0",
                                    ),
                                ],
                            ),
                            dict(
                                name="plist2",
                                entries=[
                                    dict(
                                        sequence=20,
                                        action="permit",
                                        prefix="10.0.0.0/8",
                                        eq=8,
                                    ),
                                    dict(
                                        sequence=50,
                                        action="deny",
                                        prefix="10.0.0.8/24",
                                        ge=25,
                                    ),
                                ],
                            ),
                        ],
                    ),
                    dict(
                        afi="ipv6",
                        prefix_lists=[
                            dict(
                                name="plist3",
                                description="Test plist3",
                                entries=[
                                    dict(
                                        sequence=10,
                                        action="deny",
                                        prefix="2001:db8:1000::/36",
                                        le=36,
                                    ),
                                    dict(
                                        sequence=20,
                                        action="permit",
                                        prefix="2001:db8:2000::/36",
                                    ),
                                ],
                            ),
                            dict(
                                name="plist4",
                                entries=[
                                    dict(
                                        sequence=20,
                                        action="permit",
                                        prefix="2001:db8:3000::/36",
                                    ),
                                    dict(
                                        sequence=50,
                                        action="deny",
                                        prefix="2001:db8:4000::/36",
                                    ),
                                ],
                            ),
                        ],
                    ),
                ],
                state="merged",
            ),
            ignore_provider_arg,
        )
        commands = [
            "ip prefix-list plist1 description Test plist1",
            "ip prefix-list plist1 seq 10 permit 192.168.1.0/24",
            "ip prefix-list plist1 seq 20 deny 192.168.2.0/24 mask 255.255.255.0",
            "ip prefix-list plist2 seq 20 permit 10.0.0.0/8 eq 8",
            "ip prefix-list plist2 seq 50 deny 10.0.0.8/24 ge 25",
            "ipv6 prefix-list plist3 description Test plist3",
            "ipv6 prefix-list plist3 seq 10 deny 2001:db8:1000::/36 le 36",
            "ipv6 prefix-list plist3 seq 20 permit 2001:db8:2000::/36",
            "ipv6 prefix-list plist4 seq 20 permit 2001:db8:3000::/36",
            "ipv6 prefix-list plist4 seq 50 deny 2001:db8:4000::/36",
        ]
        result = self.execute_module(changed=True)
        self.assertEqual(set(result["commands"]), set(commands))

    def test_nxos_prefix_lists_linear_merged_idempotent(self):
        # test merged for linear attributes (idempotent)
        self.get_config.return_value = dedent(
            """\
              ip prefix-list plist1 description Test plist1
              ip prefix-list plist1 seq 10 permit 192.168.1.0/24
              ip prefix-list plist1 seq 20 deny 192.168.2.0/24 mask 255.255.255.0
              ip prefix-list plist2 seq 20 permit 10.0.0.0/8 eq 8
              ip prefix-list plist2 seq 50 deny 10.0.0.8/24 ge 25
              ipv6 prefix-list plist3 description Test plist3
              ipv6 prefix-list plist3 seq 10 deny 2001:db8:1000::/36 le 36
              ipv6 prefix-list plist3 seq 20 permit 2001:db8:2000::/36
              ipv6 prefix-list plist4 seq 20 permit 2001:db8:3000::/36
              ipv6 prefix-list plist4 seq 50 deny 2001:db8:4000::/36
            """,
        )
        set_module_args(
            dict(
                config=[
                    dict(
                        afi="ipv4",
                        prefix_lists=[
                            dict(
                                name="plist1",
                                description="Test plist1",
                                entries=[
                                    dict(
                                        sequence=10,
                                        action="permit",
                                        prefix="192.168.1.0/24",
                                    ),
                                    dict(
                                        sequence=20,
                                        action="deny",
                                        prefix="192.168.2.0/24",
                                        mask="255.255.255.0",
                                    ),
                                ],
                            ),
                            dict(
                                name="plist2",
                                entries=[
                                    dict(
                                        sequence=20,
                                        action="permit",
                                        prefix="10.0.0.0/8",
                                        eq=8,
                                    ),
                                    dict(
                                        sequence=50,
                                        action="deny",
                                        prefix="10.0.0.8/24",
                                        ge=25,
                                    ),
                                ],
                            ),
                        ],
                    ),
                    dict(
                        afi="ipv6",
                        prefix_lists=[
                            dict(
                                name="plist3",
                                description="Test plist3",
                                entries=[
                                    dict(
                                        sequence=10,
                                        action="deny",
                                        prefix="2001:db8:1000::/36",
                                        le=36,
                                    ),
                                    dict(
                                        sequence=20,
                                        action="permit",
                                        prefix="2001:db8:2000::/36",
                                    ),
                                ],
                            ),
                            dict(
                                name="plist4",
                                entries=[
                                    dict(
                                        sequence=20,
                                        action="permit",
                                        prefix="2001:db8:3000::/36",
                                    ),
                                    dict(
                                        sequence=50,
                                        action="deny",
                                        prefix="2001:db8:4000::/36",
                                    ),
                                ],
                            ),
                        ],
                    ),
                ],
                state="merged",
            ),
            ignore_provider_arg,
        )
        result = self.execute_module(changed=False)
        self.assertEqual(result["commands"], [])

    def test_nxos_prefix_lists_merged_update(self):
        # test existing sequence with merged (should fail)
        self.get_config.return_value = dedent(
            """\
              ip prefix-list plist1 description Test plist1
              ip prefix-list plist1 seq 10 permit 192.168.1.0/24
              ip prefix-list plist1 seq 20 deny 192.168.2.0/24 mask 255.255.255.0
              ip prefix-list plist2 seq 20 permit 10.0.0.0/8 eq 8
              ip prefix-list plist2 seq 50 deny 10.0.0.8/24 ge 25
              ipv6 prefix-list plist3 description Test plist3
              ipv6 prefix-list plist3 seq 10 deny 2001:db8:1000::/36 le 36
              ipv6 prefix-list plist3 seq 20 permit 2001:db8:2000::/36
              ipv6 prefix-list plist4 seq 20 permit 2001:db8:3000::/36
              ipv6 prefix-list plist4 seq 50 deny 2001:db8:4000::/36
            """,
        )
        set_module_args(
            dict(
                config=[
                    dict(
                        afi="ipv4",
                        prefix_lists=[
                            dict(
                                name="plist1",
                                description="Test plist1",
                                entries=[
                                    dict(
                                        sequence=10,
                                        action="permit",
                                        prefix="192.168.8.0/24",
                                    ),
                                ],
                            ),
                        ],
                    ),
                ],
                state="merged",
            ),
            ignore_provider_arg,
        )
        result = self.execute_module(failed=True)
        self.assertEqual(
            result["msg"],
            "Cannot update existing sequence 10 of prefix list plist1 with state merged."
            " Please use state replaced or overridden.",
        )

    def test_nxos_prefix_lists_replaced_update(self):
        # test existing sequence with replaced
        self.get_config.return_value = dedent(
            """\
              ip prefix-list plist1 description Test plist1
              ip prefix-list plist1 seq 10 permit 192.168.1.0/24
              ip prefix-list plist1 seq 20 deny 192.168.2.0/24 mask 255.255.255.0
              ip prefix-list plist2 seq 20 permit 10.0.0.0/8 eq 8
              ip prefix-list plist2 seq 50 deny 10.0.0.8/24 ge 25
              ipv6 prefix-list plist3 description Test plist3
              ipv6 prefix-list plist3 seq 10 deny 2001:db8:1000::/36 le 36
              ipv6 prefix-list plist3 seq 20 permit 2001:db8:2000::/36
              ipv6 prefix-list plist4 seq 20 permit 2001:db8:3000::/36
              ipv6 prefix-list plist4 seq 50 deny 2001:db8:4000::/36
            """,
        )
        set_module_args(
            dict(
                config=[
                    dict(
                        afi="ipv4",
                        prefix_lists=[
                            dict(
                                name="plist1",
                                description="Test plist1",
                                entries=[
                                    dict(
                                        sequence=10,
                                        action="permit",
                                        prefix="192.168.8.0/24",
                                    ),
                                    dict(
                                        sequence=20,
                                        action="deny",
                                        prefix="192.168.2.0/24",
                                        mask="255.255.255.0",
                                    ),
                                ],
                            ),
                        ],
                    ),
                ],
                state="replaced",
            ),
            ignore_provider_arg,
        )
        commands = [
            "no ip prefix-list plist1 seq 10 permit 192.168.1.0/24",
            "ip prefix-list plist1 seq 10 permit 192.168.8.0/24",
        ]
        result = self.execute_module(changed=True)
        self.assertEqual(set(result["commands"]), set(commands))

    def test_nxos_prefix_lists_replaced(self):
        # test replaced
        self.get_config.return_value = dedent(
            """\
              ip prefix-list plist1 description Test plist1
              ip prefix-list plist1 seq 10 permit 192.168.1.0/24
              ip prefix-list plist1 seq 20 deny 192.168.2.0/24 mask 255.255.255.0
              ip prefix-list plist2 seq 20 permit 10.0.0.0/8 eq 8
              ip prefix-list plist2 seq 50 deny 10.0.0.8/24 ge 25
              ipv6 prefix-list plist3 description Test plist3
              ipv6 prefix-list plist3 seq 10 deny 2001:db8:1000::/36 le 36
              ipv6 prefix-list plist3 seq 20 permit 2001:db8:2000::/36
              ipv6 prefix-list plist4 seq 20 permit 2001:db8:3000::/36
              ipv6 prefix-list plist4 seq 50 deny 2001:db8:4000::/36
            """,
        )
        set_module_args(
            dict(
                config=[
                    dict(
                        afi="ipv4",
                        prefix_lists=[
                            dict(
                                name="plist1",
                                entries=[
                                    dict(
                                        sequence=10,
                                        action="permit",
                                        prefix="192.168.8.0/24",
                                    ),
                                ],
                            ),
                        ],
                    ),
                ],
                state="replaced",
            ),
            ignore_provider_arg,
        )
        commands = [
            "no ip prefix-list plist1 description Test plist1",
            "no ip prefix-list plist1 seq 20 deny 192.168.2.0/24 mask 255.255.255.0",
            "no ip prefix-list plist1 seq 10 permit 192.168.1.0/24",
            "ip prefix-list plist1 seq 10 permit 192.168.8.0/24",
        ]
        result = self.execute_module(changed=True)
        self.assertEqual(set(result["commands"]), set(commands))

    def test_nxos_prefix_lists_overridden(self):
        # test overridden
        self.get_config.return_value = dedent(
            """\
              ip prefix-list plist1 description Test plist1
              ip prefix-list plist1 seq 10 permit 192.168.1.0/24
              ip prefix-list plist1 seq 20 deny 192.168.2.0/24 mask 255.255.255.0
              ip prefix-list plist2 seq 20 permit 10.0.0.0/8 eq 8
              ip prefix-list plist2 seq 50 deny 10.0.0.8/24 ge 25
              ipv6 prefix-list plist3 description Test plist3
              ipv6 prefix-list plist3 seq 10 deny 2001:db8:1000::/36 le 36
              ipv6 prefix-list plist3 seq 20 permit 2001:db8:2000::/36
              ipv6 prefix-list plist4 seq 20 permit 2001:db8:3000::/36
              ipv6 prefix-list plist4 seq 50 deny 2001:db8:4000::/36
            """,
        )
        set_module_args(
            dict(
                config=[
                    dict(
                        afi="ipv4",
                        prefix_lists=[
                            dict(
                                name="plist1",
                                entries=[
                                    dict(
                                        sequence=10,
                                        action="permit",
                                        prefix="192.168.8.0/24",
                                    ),
                                ],
                            ),
                        ],
                    ),
                ],
                state="overridden",
            ),
            ignore_provider_arg,
        )
        commands = [
            "no ip prefix-list plist1 description Test plist1",
            "no ip prefix-list plist1 seq 20 deny 192.168.2.0/24 mask 255.255.255.0",
            "no ip prefix-list plist1 seq 10 permit 192.168.1.0/24",
            "ip prefix-list plist1 seq 10 permit 192.168.8.0/24",
            "no ipv6 prefix-list plist4",
            "no ip prefix-list plist2",
            "no ipv6 prefix-list plist3",
        ]
        result = self.execute_module(changed=True)
        self.assertEqual(set(result["commands"]), set(commands))

    def test_nxos_prefix_lists_deleted_afi(self):
        # test deleted (AFI)
        self.get_config.return_value = dedent(
            """\
              ip prefix-list plist1 description Test plist1
              ip prefix-list plist1 seq 10 permit 192.168.1.0/24
              ip prefix-list plist1 seq 20 deny 192.168.2.0/24 mask 255.255.255.0
              ip prefix-list plist2 seq 20 permit 10.0.0.0/8 eq 8
              ip prefix-list plist2 seq 50 deny 10.0.0.8/24 ge 25
              ipv6 prefix-list plist3 description Test plist3
              ipv6 prefix-list plist3 seq 10 deny 2001:db8:1000::/36 le 36
              ipv6 prefix-list plist3 seq 20 permit 2001:db8:2000::/36
              ipv6 prefix-list plist4 seq 20 permit 2001:db8:3000::/36
              ipv6 prefix-list plist4 seq 50 deny 2001:db8:4000::/36
            """,
        )
        set_module_args(
            dict(config=[dict(afi="ipv4")], state="deleted"),
            ignore_provider_arg,
        )
        commands = ["no ip prefix-list plist1", "no ip prefix-list plist2"]
        result = self.execute_module(changed=True)
        self.assertEqual(set(result["commands"]), set(commands))

    def test_nxos_prefix_lists_deleted_prefix_list(self):
        # test deleted (prefix-list)
        self.get_config.return_value = dedent(
            """\
              ip prefix-list plist1 description Test plist1
              ip prefix-list plist1 seq 10 permit 192.168.1.0/24
              ip prefix-list plist1 seq 20 deny 192.168.2.0/24 mask 255.255.255.0
              ip prefix-list plist2 seq 20 permit 10.0.0.0/8 eq 8
              ip prefix-list plist2 seq 50 deny 10.0.0.8/24 ge 25
              ipv6 prefix-list plist3 description Test plist3
              ipv6 prefix-list plist3 seq 10 deny 2001:db8:1000::/36 le 36
              ipv6 prefix-list plist3 seq 20 permit 2001:db8:2000::/36
              ipv6 prefix-list plist4 seq 20 permit 2001:db8:3000::/36
              ipv6 prefix-list plist4 seq 50 deny 2001:db8:4000::/36
            """,
        )
        set_module_args(
            dict(
                config=[dict(afi="ipv6", prefix_lists=[dict(name="plist3")])],
                state="deleted",
            ),
            ignore_provider_arg,
        )
        commands = ["no ipv6 prefix-list plist3"]
        result = self.execute_module(changed=True)
        self.assertEqual(set(result["commands"]), set(commands))

    def test_nxos_prefix_lists_deleted_all(self):
        # test deleted
        self.get_config.return_value = dedent(
            """\
              ip prefix-list plist1 description Test plist1
              ip prefix-list plist1 seq 10 permit 192.168.1.0/24
              ip prefix-list plist1 seq 20 deny 192.168.2.0/24 mask 255.255.255.0
              ip prefix-list plist2 seq 20 permit 10.0.0.0/8 eq 8
              ip prefix-list plist2 seq 50 deny 10.0.0.8/24 ge 25
              ipv6 prefix-list plist3 description Test plist3
              ipv6 prefix-list plist3 seq 10 deny 2001:db8:1000::/36 le 36
              ipv6 prefix-list plist3 seq 20 permit 2001:db8:2000::/36
              ipv6 prefix-list plist4 seq 20 permit 2001:db8:3000::/36
              ipv6 prefix-list plist4 seq 50 deny 2001:db8:4000::/36
            """,
        )
        set_module_args(dict(state="deleted"), ignore_provider_arg)
        commands = [
            "no ip prefix-list plist1",
            "no ip prefix-list plist2",
            "no ipv6 prefix-list plist3",
            "no ipv6 prefix-list plist4",
        ]
        result = self.execute_module(changed=True)
        self.assertEqual(set(result["commands"]), set(commands))

    def test_nxos_prefix_lists_parsed(self):
        # test parsed
        cfg = dedent(
            """\
              ip prefix-list plist1 description Test plist1
              ipv6 prefix-list plist3 description Test plist3
            """,
        )
        set_module_args(dict(running_config=cfg, state="parsed"), ignore_provider_arg)
        parsed = [
            {
                "afi": "ipv4",
                "prefix_lists": [{"name": "plist1", "description": "Test plist1"}],
            },
            {
                "afi": "ipv6",
                "prefix_lists": [{"name": "plist3", "description": "Test plist3"}],
            },
        ]
        result = self.execute_module(changed=False)
        self.assertEqual(result["parsed"], parsed)

    def test_nxos_prefix_lists_gathered(self):
        # test gathered
        self.get_config.return_value = dedent(
            """\
              ip prefix-list plist1 description Test plist1
              ip prefix-list plist1 seq 20 deny 192.168.2.0/24 mask 255.255.255.0
              ipv6 prefix-list plist3 description Test plist3
              ipv6 prefix-list plist3 seq 50 deny 2001:db8:4000::/36
            """,
        )
        set_module_args(dict(state="gathered"), ignore_provider_arg)
        gathered = [
            {
                "afi": "ipv4",
                "prefix_lists": [
                    {
                        "name": "plist1",
                        "description": "Test plist1",
                        "entries": [
                            {
                                "sequence": 20,
                                "action": "deny",
                                "prefix": "192.168.2.0/24",
                                "mask": "255.255.255.0",
                            },
                        ],
                    },
                ],
            },
            {
                "afi": "ipv6",
                "prefix_lists": [
                    {
                        "name": "plist3",
                        "description": "Test plist3",
                        "entries": [
                            {
                                "sequence": 50,
                                "action": "deny",
                                "prefix": "2001:db8:4000::/36",
                            },
                        ],
                    },
                ],
            },
        ]
        result = self.execute_module(changed=False)
        self.assertEqual(result["gathered"], gathered)



#!/usr/bin/python
# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type


DOCUMENTATION = '''
---
module: ec2_asg_instance_refresh_info
version_added: 3.2.0
short_description: Gather information about ec2 Auto Scaling Group (ASG) Instance Refreshes in AWS
description:
  - Describes one or more instance refreshes.
  - You can determine the status of a request by looking at the I(status) parameter.
author: "Dan Khersonsky (@danquixote)"
options:
  name:
    description:
      - The name of the Auto Scaling group.
    type: str
    required: true
  ids:
    description:
      - One or more instance refresh IDs.
    type: list
    elements: str
    default: []
  next_token:
    description:
      - The token for the next set of items to return. (You received this token from a previous call.)
    type: str
  max_records:
    description:
      - The maximum number of items to return with this call. The default value is 50 and the maximum value is 100.
    type: int
    required: false
extends_documentation_fragment:
- amazon.aws.aws
- amazon.aws.ec2

'''

EXAMPLES = '''
# Note: These examples do not set authentication details, see the AWS Guide for details.

- name: Find an refresh by ASG name
  community.aws.ec2_asg_instance_refresh_info:
    name: somename-asg

- name: Find an refresh by ASG name and one or more refresh-IDs
  community.aws.ec2_asg_instance_refresh_info:
    name: somename-asg
    ids: ['some-id-123']
  register: asgs

- name: Find an refresh by ASG name and set max_records
  community.aws.ec2_asg_instance_refresh_info:
    name: somename-asg
    max_records: 4
  register: asgs

- name: Find an refresh by ASG name and NextToken, if received from a previous call
  community.aws.ec2_asg_instance_refresh_info:
    name: somename-asg
    next_token: 'some-token-123'
  register: asgs
'''

RETURN = '''
---
instance_refresh_id:
    description: instance refresh id
    returned: success
    type: str
    sample: "08b91cf7-8fa6-48af-b6a6-d227f40f1b9b"
auto_scaling_group_name:
    description: Name of autoscaling group
    returned: success
    type: str
    sample: "public-webapp-production-1"
status:
    description:
      -  The current state of the group when DeleteAutoScalingGroup is in progress.
      -  The following are the possible statuses
      -    Pending --  The request was created, but the operation has not started.
      -    InProgress --  The operation is in progress.
      -    Successful --  The operation completed successfully.
      -    Failed --  The operation failed to complete. You can troubleshoot using the status reason and the scaling activities.
      -    Cancelling --
      -        An ongoing operation is being cancelled.
      -        Cancellation does not roll back any replacements that have already been completed,
      -        but it prevents new replacements from being started.
      -    Cancelled --  The operation is cancelled.
    returned: success
    type: str
    sample: "Pending"
start_time:
    description: The date and time this ASG was created, in ISO 8601 format.
    returned: success
    type: str
    sample: "2015-11-25T00:05:36.309Z"
end_time:
    description: The date and time this ASG was created, in ISO 8601 format.
    returned: success
    type: str
    sample: "2015-11-25T00:05:36.309Z"
percentage_complete:
    description: the % of completeness
    returned: success
    type: int
    sample: 100
instances_to_update:
    description: num. of instance to update
    returned: success
    type: int
    sample: 5
'''

try:
    from botocore.exceptions import BotoCoreError, ClientError
except ImportError:
    pass  # caught by AnsibleAWSModule

from ansible_collections.amazon.aws.plugins.module_utils.core import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import AWSRetry
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import camel_dict_to_snake_dict


def find_asg_instance_refreshes(conn, module):
    """
    Args:
        conn (boto3.AutoScaling.Client): Valid Boto3 ASG client.
        module: AnsibleAWSModule object

    Returns:
        {
            "instance_refreshes": [
                    {
                        'auto_scaling_group_name': 'ansible-test-hermes-63642726-asg',
                        'instance_refresh_id': '6507a3e5-4950-4503-8978-e9f2636efc09',
                        'instances_to_update': 1,
                        'percentage_complete': 0,
                        "preferences": {
                            "instance_warmup": 60,
                            "min_healthy_percentage": 90,
                            "skip_matching": false
                        },
                        'start_time': '2021-02-04T03:39:40+00:00',
                        'status': 'Cancelled',
                        'status_reason': 'Cancelled due to user request.',
                    }
            ],
            'next_token': 'string'
        }
        """

    asg_name = module.params.get('name')
    asg_ids = module.params.get('ids')
    asg_next_token = module.params.get('next_token')
    asg_max_records = module.params.get('max_records')

    args = {}
    args['AutoScalingGroupName'] = asg_name
    if asg_ids:
        args['InstanceRefreshIds'] = asg_ids
    if asg_next_token:
        args['NextToken'] = asg_next_token
    if asg_max_records:
        args['MaxRecords'] = asg_max_records

    try:
        instance_refreshes_result = {}
        response = conn.describe_instance_refreshes(**args)
        if 'InstanceRefreshes' in response:
            instance_refreshes_dict = dict(
                instance_refreshes=response['InstanceRefreshes'], next_token=response.get('next_token', ''))
            instance_refreshes_result = camel_dict_to_snake_dict(
                instance_refreshes_dict)

        while 'NextToken' in response:
            args['NextToken'] = response['NextToken']
            response = conn.describe_instance_refreshes(**args)
            if 'InstanceRefreshes' in response:
                instance_refreshes_dict = camel_dict_to_snake_dict(dict(
                    instance_refreshes=response['InstanceRefreshes'], next_token=response.get('next_token', '')))
                instance_refreshes_result.update(instance_refreshes_dict)

        return module.exit_json(**instance_refreshes_result)
    except (BotoCoreError, ClientError) as e:
        module.fail_json_aws(e, msg='Failed to describe InstanceRefreshes')


def main():

    argument_spec = dict(
        name=dict(required=True, type='str'),
        ids=dict(required=False, default=[], elements='str', type='list'),
        next_token=dict(required=False, default=None, type='str', no_log=True),
        max_records=dict(required=False, type='int'),
    )

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )

    autoscaling = module.client(
        'autoscaling',
        retry_decorator=AWSRetry.jittered_backoff(retries=10)
    )
    find_asg_instance_refreshes(autoscaling, module)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2022 Western Digital Corporation
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: wdc_redfish_info
short_description: Manages WDC UltraStar Data102 Out-Of-Band controllers using Redfish APIs
version_added: 5.4.0
description:
  - Builds Redfish URIs locally and sends them to remote OOB controllers to
    get information back.
options:
  category:
    required: true
    description:
      - Category to execute on OOB controller.
    type: str
  command:
    required: true
    description:
      - List of commands to execute on OOB controller.
    type: list
    elements: str
  baseuri:
    description:
      - Base URI of OOB controller.  Must include this or I(ioms).
    type: str
  ioms:
    description:
      - List of IOM FQDNs for the enclosure.  Must include this or I(baseuri).
    type: list
    elements: str
  username:
    description:
      - User for authentication with OOB controller.
    type: str
  password:
    description:
      - Password for authentication with OOB controller.
    type: str
  auth_token:
    description:
      - Security token for authentication with OOB controller.
    type: str
  timeout:
    description:
      - Timeout in seconds for URL requests to OOB controller.
    default: 10
    type: int

notes:
  - In the inventory, you can specify baseuri or ioms.  See the EXAMPLES section.
  - ioms is a list of FQDNs for the enclosure's IOMs.

author: Mike Moerk (@mikemoerk)
'''

EXAMPLES = '''
- name: Get Simple Update Status with individual IOMs specified
  community.general.wdc_redfish_info:
    category: Update
    command: SimpleUpdateStatus
    ioms:
      - iom1.wdc.com
      - iom2.wdc.com
    username: "{{ username }}"
    password: "{{ password }}"
  register: result

- name: Print fetched information
  ansible.builtin.debug:
    msg: "{{ result.redfish_facts.simple_update_status.entries | to_nice_json }}"

- name: Get Simple Update Status with baseuri specified
  community.general.wdc_redfish_info:
    category: Update
    command: SimpleUpdateStatus
    baseuri: "iom1.wdc.com"
    username: "{{ username }}"
    password: "{{ password }}"
  register: result

- name: Print fetched information
  ansible.builtin.debug:
    msg: "{{ result.redfish_facts.simple_update_status.entries | to_nice_json }}"
'''

RETURN = '''
Description:
    description: Firmware update status description.
    returned: always
    type: str
    sample:
      - Ready for FW update
      - FW update in progress
      - FW update completed. Waiting for activation.
ErrorCode:
    description: Numeric error code for firmware update status.  Non-zero indicates an error condition.
    returned: always
    type: int
    sample:
      - 0
EstimatedRemainingMinutes:
    description: Estimated number of minutes remaining in firmware update operation.
    returned: always
    type: int
    sample:
      - 0
      - 20
StatusCode:
    description: Firmware update status code.
    returned: always
    type: int
    sample:
      - 0 (Ready for FW update)
      - 1 (FW update in progress)
      - 2 (FW update completed.  Waiting for activation.)
      - 3 (FW update failed.)
'''

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.common.text.converters import to_native
from ansible_collections.community.general.plugins.module_utils.wdc_redfish_utils import WdcRedfishUtils

CATEGORY_COMMANDS_ALL = {
    "Update": ["SimpleUpdateStatus"]
}


def main():
    result = {}
    module = AnsibleModule(
        argument_spec=dict(
            category=dict(required=True),
            command=dict(required=True, type='list', elements='str'),
            ioms=dict(type='list', elements='str'),
            baseuri=dict(),
            username=dict(),
            password=dict(no_log=True),
            auth_token=dict(no_log=True),
            timeout=dict(type='int', default=10)
        ),
        required_together=[
            ('username', 'password'),
        ],
        required_one_of=[
            ('username', 'auth_token'),
            ('baseuri', 'ioms')
        ],
        mutually_exclusive=[
            ('username', 'auth_token'),
        ],
        supports_check_mode=True
    )

    category = module.params['category']
    command_list = module.params['command']

    # admin credentials used for authentication
    creds = {'user': module.params['username'],
             'pswd': module.params['password'],
             'token': module.params['auth_token']}

    # timeout
    timeout = module.params['timeout']

    # Check that Category is valid
    if category not in CATEGORY_COMMANDS_ALL:
        module.fail_json(msg=to_native("Invalid Category '%s'. Valid Categories = %s" % (category, sorted(CATEGORY_COMMANDS_ALL.keys()))))

    # Check that all commands are valid
    for cmd in command_list:
        # Fail if even one command given is invalid
        if cmd not in CATEGORY_COMMANDS_ALL[category]:
            module.fail_json(msg=to_native("Invalid Command '%s'. Valid Commands = %s" % (cmd, CATEGORY_COMMANDS_ALL[category])))

    # Build root URI(s)
    if module.params.get("baseuri") is not None:
        root_uris = ["https://" + module.params['baseuri']]
    else:
        root_uris = [
            "https://" + iom for iom in module.params['ioms']
        ]
    rf_utils = WdcRedfishUtils(creds, root_uris, timeout, module,
                               resource_id=None,
                               data_modification=False
                               )

    # Organize by Categories / Commands

    if category == "Update":
        # execute only if we find UpdateService resources
        resource = rf_utils._find_updateservice_resource()
        if resource['ret'] is False:
            module.fail_json(msg=resource['msg'])
        for command in command_list:
            if command == "SimpleUpdateStatus":
                simple_update_status_result = rf_utils.get_simple_update_status()
                if simple_update_status_result['ret'] is False:
                    module.fail_json(msg=to_native(result['msg']))
                else:
                    del simple_update_status_result['ret']
                    result["simple_update_status"] = simple_update_status_result
                    module.exit_json(changed=False, redfish_facts=result)


if __name__ == '__main__':
    main()
